
PROJECTNAME = "PFGDataGrid"

DEFAULT_ADD_CONTENT_PERMISSION = "PloneFormGen: Add Content"

GLOBALS = globals()

try:
    from Products.PloneFormGen.content import fieldsBase
    HAVE_PFG = True
except ImportError:
    HAVE_PFG = False




from Products.Archetypes.public import process_types, listTypes
from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory

from config import *

#registerDirectory(SKINS_DIR, GLOBALS)

def initialize(context):
    import datagridff

    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)

    utils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = DEFAULT_ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)



""" PloneFormGen, basic Zope Product Initialization """

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'


import logging
logger = logging.getLogger("PloneFormGen")


# let's fail immediately and decisively if our dependencies
# aren't installed
try:
    from Products import PythonField, TALESField, TemplateFields
except ImportError:
    logger.error("PloneFormGen requires the ScriptableFields bundle. See PloneFormGen's README.txt.")
    raise

from Products.Archetypes.public import process_types, listTypes
from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory
from AccessControl import ModuleSecurityInfo

from Products.PloneFormGen.config import PROJECTNAME, \
    ADD_CONTENT_PERMISSION, CSA_ADD_CONTENT_PERMISSION, \
    MA_ADD_CONTENT_PERMISSION, SDA_ADD_CONTENT_PERMISSION, \
    SKINS_DIR, GLOBALS

registerDirectory(SKINS_DIR + '/PloneFormGen', GLOBALS)

def initialize(context):    

    import content, validators, tools, widgets

    # Add our tools
    utils.ToolInit('PloneFormGen Tool',
        tools=( tools.formGenTool.FormGenTool, ), 
        icon='Form.gif',
        ).initialize(context) 


    ##########
    # Add our content types
    # A little different from the average Archetype product
    # due to the need to individualize some add permissions.
    #
    # This approach borrowed from ATContentTypes
    #
    listOfTypes = listTypes(PROJECTNAME)

    content_types, constructors, ftis = process_types(
        listOfTypes,
        PROJECTNAME)
    allTypes = zip(content_types, constructors)
    for atype, constructor in allTypes:
        kind = "%s: %s" % (PROJECTNAME, atype.archetype_name)
        if atype.portal_type == 'FormCustomScriptAdapter':
            permission = CSA_ADD_CONTENT_PERMISSION
        elif atype.portal_type == 'FormMailerAdapter':
            permission = MA_ADD_CONTENT_PERMISSION
        elif atype.portal_type == 'FormSaveDataAdapter':
            permission = SDA_ADD_CONTENT_PERMISSION
        else:
            permission = ADD_CONTENT_PERMISSION
        utils.ContentInit(
            kind,
            content_types      = (atype,),
            permission         = permission,
            extra_constructors = (constructor,),
            fti                = ftis,
            ).initialize(context)


    ModuleSecurityInfo('Products.PloneFormGen').declarePublic('PloneFormGenMessageFactory')
    ModuleSecurityInfo('Products.PloneFormGen').declarePublic('HAS_PLONE25')


# Import "PloneFormGenMessageFactory as _" to create message ids
# in the ploneformgen domain
# Zope 3.1-style messagefactory module
# BBB: Zope 2.8 / Zope X3.0
try:
    from zope.i18nmessageid import MessageFactory
except ImportError:
    from messagefactory_ import PloneFormGenMessageFactory
else:
    PloneFormGenMessageFactory = MessageFactory('ploneformgen')

# Check for Plone versions
try:
    from Products.CMFPlone.migrations import v2_5
except ImportError:
    HAS_PLONE25 = False
else:
    HAS_PLONE25 = True
try:
    from Products.CMFPlone.migrations import v3_0
except ImportError:
    HAS_PLONE30 = False
else:
    HAS_PLONE30 = True




from Products.validation import validation, interfaces

from Products.CMFPlone.utils import safe_hasattr

class MaxLengthValidator:
    """ Validates whether or not a value's length is at or under
        maxlength. maxlength may come from initialization,
        from the kwargs in a call or from widget attributes.
    """


    __implements__ = (interfaces.ivalidator,)

    name = 'MaxLengthValidator'

    def __init__(self, name, maxlength=255, title='', description=''):
        self.name = name
        self.maxlength = maxlength
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):

        #import pdb; pdb.set_trace()
            
        field = kwargs.get('field', None)
        widget = getattr(field, 'widget', None)

        # get maxlength
        if kwargs.has_key('maxlength'):
            maxlength = kwargs.get('maxlength')
        elif safe_hasattr(widget, 'maxlength'):
            maxlength = int(widget.maxlength or 0)
        else:
            # set to given default value (default defaults to 0)
            maxlength = self.maxlength

        if maxlength == 0:
            return 1

        nval = len(value)
                    
        if nval <= maxlength:
            return 1
        else:
            return ("Validation failed(%(name)s): '%(value)s' is too long. Must be no longer than %(max)s characters." %
                    { 'name' : self.name, 'value': getattr(widget, 'label', 'Entry'), 'max' : maxlength,})

validation.register(MaxLengthValidator('isNotTooLong'))



"""fieldset -- A fieldset container for form fields"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from types import BooleanType

from zope.interface import implements, providedBy

import logging

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

from Products.CMFPlone.utils import safe_hasattr, base_hasattr

from Products.Archetypes.public import *

from Products.ATContentTypes.content.folder import ATFolderSchema, ATFolder
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.base import registerATCT

from Products.PloneFormGen.config import *
from Products.PloneFormGen.widgets import \
    FieldsetStartWidget, FieldsetEndWidget

from Products.PloneFormGen import PloneFormGenMessageFactory as _
from Products.PloneFormGen import HAS_PLONE25

if HAS_PLONE25:
  import zope.i18n

FieldsetFolderSchema = ATFolderSchema.copy() + Schema((
    BooleanField('useLegend',
        required=0,
        searchable=0,
        default='1',
        widget=BooleanWidget(label='Show Title as Legend',
            label_msgid = "label_showlegend_text",
            description_msgid = "help_showlegend_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    ))

FieldsetFolderSchema['description'].widget.label = 'Fieldset Help'
FieldsetFolderSchema['description'].widget.i18n_domain = 'ploneformgen'
FieldsetFolderSchema['description'].widget.label_msgid = 'label_fieldsethelp_text'
FieldsetFolderSchema['description'].widget.description = None
FieldsetFolderSchema['description'].widget.description_msgid = None
FieldsetFolderSchema.moveField('description', after='useLegend')


class FieldsetFolder(ATFolder):
    """A folder which groups form fields as a fieldset."""

    schema         =  FieldsetFolderSchema

    content_icon   = 'Fieldset.gif'
    meta_type      = 'FieldsetFolder'
    portal_type    = 'FieldsetFolder'
    archetype_name = 'Fieldset Folder'
    suppl_views = ()

    typeDescription= 'A folder which groups form fields as a fieldset.'

    global_allow = 0    

    # XXX We should do this with a tool so that others may add fields
    allowed_content_types = fieldTypes

    security       = ClassSecurityInfo()


    def __init__(self, oid, **kwargs):
        """ initialize class """

        ATFolder.__init__(self, oid, **kwargs)
        
        self.fsStartField = StringField('FieldSetStart',
            searchable=0,
            required=0,
            write_permission = View,
            widget=FieldsetStartWidget(),
            )
        

        self.fsEndField = StringField('FieldSetEnd',
            searchable=0,
            required=0,
            write_permission = View,
            widget=FieldsetEndWidget(),
            )
        

    security.declareProtected(ModifyPortalContent, 'setTitle')
    def setTitle(self, value, **kw):
        """ set title of object and field label """

        self.title = value
        self.fsStartField.widget.label = value


    security.declareProtected(ModifyPortalContent, 'setDescription')
    def setDescription(self, value, **kw):
        """ set description for field widget """

        self.fsStartField.widget.description = value
        self.getField('description').set(self, value, **kw)


    security.declareProtected(ModifyPortalContent, 'setDescription')
    def setUseLegend(self, value, **kw):
        """ set useLegend as attribute and widget attribute """

        if type(value) == BooleanType:
            self.fsStartField.widget.show_legend = value
        else:
            self.fsStartField.widget.show_legend = value == '1'

        self.useLegend = value


    security.declarePrivate('fieldsetFields')
    def fieldsetFields(self, objTypes=None, includeFSMarkers=False):
        """ 
        return list of enclosed fields;
        if includeFSMarkers, include markers for fieldset start/end
        """

        myObjs = []
        if includeFSMarkers:
            myObjs.append(self.fsStartField)

        for obj in self.objectValues(objTypes):
            # use shasattr to make sure we're not aquiring
            # fgField by acquisition
            if base_hasattr(obj, 'fgField'):
                myObjs.append(obj)

        if includeFSMarkers:
            myObjs.append(self.fsEndField)

        return myObjs


    security.declareProtected(ModifyPortalContent, 'setId')
    def setId(self, value):
        """Sets the object id. Changes both object and field id.
        """

        badIds = (
            'language',
            'form',
            'form_submit',
            'fieldset',
            'last_referer',
            'add_reference',
            )

        if value in badIds:
            raise BadRequest, 'The id "%s" is reserved.' % value

        ATFolder.setId(self, value)
        self.fsStartField.__name__ = self.getId()


    def manage_afterAdd(self, item, container):
        # XXX TODO: when we're done with 2.1.x, implement this via event subscription

        ATFolder.manage_afterAdd(self, item, container)

        id = self.getId()        
        if self.fsStartField.__name__ != id:
            self.fsStartField.__name__ = id


    # security is inherited
    def checkIdAvailable(self, id):
        """ Checks for good id by asking form folder """
        
        return self.formFolderObject().checkIdAvailable(id)


registerATCT(FieldsetFolder, PROJECTNAME)



"""FormFolder -- A container for form fields"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from zope.interface import implements, providedBy

import logging

from AccessControl import ClassSecurityInfo, Unauthorized
from Products.CMFCore.permissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.Expression import getExprContext

from Products.CMFPlone.utils import safe_hasattr

try:
    from Products.LinguaPlone.public import *
except ImportError:
    from Products.Archetypes.public import *

from Products.Archetypes.utils import shasattr, getRelURL
from Products.Archetypes.interfaces.field import IField

from Products.ATContentTypes.content.folder import ATFolderSchema, ATFolder
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.configuration import zconf

from Products.TALESField import TALESString

from Products.PloneFormGen.interfaces import \
    IPloneFormGenForm, IPloneFormGenActionAdapter, IPloneFormGenThanksPage
from Products.PloneFormGen.config import \
    PROJECTNAME, fieldTypes, adapterTypes, thanksTypes, fieldsetTypes, \
    EDIT_TALES_PERMISSION, EDIT_ADVANCED_PERMISSION, BAD_IDS, FORM_ERROR_MARKER
from Products.PloneFormGen.content import validationMessages

from Products.PloneFormGen import PloneFormGenMessageFactory as _
from Products.PloneFormGen import HAS_PLONE25, HAS_PLONE30

from types import StringTypes

if HAS_PLONE25:
  import zope.i18n

logger = logging.getLogger("PloneFormGen")    

FormFolderSchema = ATFolderSchema.copy() + Schema((
    StringField('submitLabel',
        required=0,
        searchable=0,
        default="Submit",
        widget=StringWidget(
            label="Submit Button Label",
            label_msgid = "label_submitlabel_text",
            description_msgid = "help_submitlabel_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    BooleanField('useCancelButton',
        required=0,
        searchable=0,
        default='0',
        languageIndependent=1,
        widget=BooleanWidget(label='Show Reset Button',
            label_msgid = "label_showcancel_text",
            description_msgid = "help_showcancel_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    StringField('resetLabel',
        required=0,
        searchable=0,
        default="Reset",
        widget=StringWidget(
                label="Reset Button Label",
                label_msgid = "label_reset_button",
                i18n_domain = 'ploneformgen',
                ),
        ),    
    LinesField('actionAdapter',
        vocabulary='actionAdaptersDL',
        widget=MultiSelectionWidget(
            label="Action Adapter",
            description="""
                To make your form do something useful when submitted:
                add one or more form action adapters to the form folder,
                configure them, then return to this
                form and select the active ones.
                """,
            format='checkbox',
            label_msgid = "label_actionadapter_text",
            description_msgid = "help_actionadapter_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    StringField('thanksPage',
        searchable=False,
        required=False,
        vocabulary='thanksPageVocabulary',
        widget=SelectionWidget(
            label='Thanks Page',
            label_msgid = "label_thankspage_text",
            description="""
                Pick a contained page you wish to show on a successful
                form submit. (If none are available, add one.) Choose none to simply display the form
                field values.
            """,
            description_msgid = "help_thankspage_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    TextField('formPrologue',
        schemata='decoration',
        required=False,
        # Disable search to bypass a unicode decode error in portal_catalog indexes.
        searchable=False,
        primary=False,
        validators = ('isTidyHtmlWithCleanup',),
        default_content_type = zconf.ATDocument.default_content_type,
        default_output_type = 'text/x-html-safe',
        allowable_content_types = zconf.ATDocument.allowed_content_types,
        widget = RichWidget(
            label = "Form Prologue",
            label_msgid = "label_prologue_text",
            description = "This text will be displayed above the form fields.",
            description_msgid = "help_prologue_text",
            rows = 8,
            i18n_domain = "ploneformgen",
            allow_file_upload = zconf.ATDocument.allow_document_upload,
            ),
        ),
    TextField('formEpilogue',
        schemata='decoration',
        required=False,
        # Disable search to bypass a unicode decode error in portal_catalog indexes.
        searchable=False,
        primary=False,
        validators = ('isTidyHtmlWithCleanup',),
        default_content_type = zconf.ATDocument.default_content_type,
        default_output_type = 'text/x-html-safe',
        allowable_content_types = zconf.ATDocument.allowed_content_types,
        widget = RichWidget(
            label = "Form Epilogue",
            label_msgid = "label_epilogue_text",
            description = "The text will be displayed after the form fields.",
            description_msgid = "help_epilogue_text",
            rows = 8,
            i18n_domain = "ploneformgen",
            allow_file_upload = zconf.ATDocument.allow_document_upload,
            ),
        ),
    StringField('thanksPageOverride',
        schemata='overrides',
        searchable=0,
        required=0,
        languageIndependent=1,
        write_permission=EDIT_TALES_PERMISSION,
        widget=StringWidget(label="Custom Success Action",
            description="""
                Use this field in place of a thanks-page designation
                to determine final action after calling
                your action adapter (if you have one). You would usually use this for a custom
                success template or script.
                Leave empty if unneeded. Otherwise, specify as you would a CMFFormController
                action type and argument,
                complete with type of action to execute (e.g., "redirect_to" or "traverse_to")
                and a TALES expression. For example, "redirect_to:string:thanks-page" would
                redirect to 'thanks-page'.
            """,
            size=70,
            label_msgid = "label_thankspageoverride_text",
            description_msgid = "help_thankspageoverride_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    StringField('formActionOverride',
        schemata='overrides',
        searchable=0,
        required=0,
        write_permission=EDIT_ADVANCED_PERMISSION,
        validators='isURL',
        languageIndependent=1,
        widget=StringWidget(label="Custom Form Action",
            description="""
                Use this field to override the form action attribute.
                Specify a URL to which the form will post.
                This will bypass form validation, success action
                adapter and thanks page.
            """,
            size=70,
            label_msgid = "label_formactionoverride_text",
            description_msgid = "help_formactionoverride_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    TALESString('onDisplayOverride',
        schemata='overrides',
        searchable=0,
        required=0,
        validators=('talesvalidator',),
        write_permission=EDIT_TALES_PERMISSION,
        default='',
        languageIndependent=1,
        widget=StringWidget(label="Form Setup Script",
            description="""
                A TALES expression that will be called when the form is displayed.
                Leave empty if unneeded.
                The most common use of this field is to call a python script that sets
                defaults for multiple fields by pre-populating request.form.
                Any value returned by the expression is ignored.
                PLEASE NOTE: errors in the evaluation of this expression will cause
                an error on form display.
            """,
            size=70,
            i18n_domain = "ploneformgen",
            label_msgid = "label_OnDisplayOverride_text",
            description_msgid = "help_OnDisplayOverride_text",
            ),
        ),
    TALESString('afterValidationOverride',
        schemata='overrides',
        searchable=0,
        required=0,
        validators=('talesvalidator',),
        write_permission=EDIT_TALES_PERMISSION,
        default='',
        languageIndependent=1,
        widget=StringWidget(label="After Validation Script",
            description="""
                A TALES expression that will be called after the form is 
                successfully validated, but before calling an action adapter
                (if any) or displaying a thanks page.
                Form input will be in the request.form dictionary.
                Leave empty if unneeded.
                The most common use of this field is to call a python script
                to clean up form input or to script an alternative action.
                Any value returned by the expression is ignored.
                PLEASE NOTE: errors in the evaluation of this expression will cause
                an error on form display.
            """,
            size=70,
            i18n_domain = "ploneformgen",
            label_msgid = "label_AfterValidationOverride_text",
            description_msgid = "help_AfterValidationOverride_text",
            ),
        ),
    TALESString('headerInjection',
        schemata='overrides',
        searchable=0,
        required=0,
        validators=('talesvalidator',),
        write_permission=EDIT_TALES_PERMISSION,
        default='',
        languageIndependent=1,
        widget=StringWidget(label="Header Injection",
            description="""
                This override field allows you to insert content into the xhtml
                head. The typical use is to add custom CSS or JavaScript.
                Specify a TALES expression returning a string. The string will
                be inserted with no interpretation.
                PLEASE NOTE: errors in the evaluation of this expression will cause
                an error on form display.
            """,
            size=70,
            i18n_domain = "ploneformgen",
            label_msgid = "label_headerInjection_text",
            description_msgid = "help_headerInjection_text",
            ),
        ),
    ))

if HAS_PLONE30:
    # Plone 3 switches the schema tab widget to a select box when
    # the number of schemata is > 6. IMHO, this has worse usability
    # than packing the default schema.
    # Also, as of P3.0, rich text fields on non-default schema
    # still don't function.
    FormFolderSchema['formPrologue'].schemata = 'default'
    FormFolderSchema['formEpilogue'].schemata = 'default'

#finalizeATCTSchema(ATFolderSchema, folderish=True, moveDiscussion=False)

class FormFolder(ATFolder):
    """A folder which can contain form fields."""

    implements(IPloneFormGenForm)

    schema         =  FormFolderSchema

    content_icon   = 'Form.gif'
    meta_type      = 'FormFolder'
    portal_type    = 'FormFolder'
    archetype_name = 'Form Folder'
    if HAS_PLONE30:
        default_view   = immediate_view = 'fg_base_view_p3'
    else:
        default_view   = immediate_view = 'fg_base_view'
    suppl_views = ()

    typeDescription= 'A folder which creates a form view from contained form fields.'

    # XXX We should do this with a tool so that others may add fields
    allowed_content_types = fieldTypes + adapterTypes + thanksTypes + fieldsetTypes + ('Document', 'Image')

    security       = ClassSecurityInfo()


    security.declarePrivate('_getFieldObjects')
    def _getFieldObjects(self, objTypes=None, includeFSMarkers=False):
        """ return list of enclosed fields """

        # This function currently checks to see if
        # an object is a form field by looking to see
        # if it has an fgField attribute.

        # Make sure we look through fieldsets
        if objTypes is not None:
            objTypes = list(objTypes)[:]
            objTypes.append('FieldsetFolder')

        myObjs = []

        for obj in self.objectValues(objTypes):
            # use shasattr to make sure we're not aquiring
            # fgField by acquisition

            # TODO: If I stick with this scheme for enable overrides,
            # I'm probably going to want to find a way to cache the result
            # in the request. _getFieldObjects potentially gets called
            # several times in a request.

            # first, see if the field enable override is set
            if shasattr(obj, 'fgTEnabled') and obj.getRawFgTEnabled():
                # process the override enabled TALES expression
                # create a context for expression evaluation
                context = getExprContext(self, obj)
                # call the tales expression, passing our custom context
                enabled = obj.getFgTEnabled(expression_context=context)
            else:
                enabled = True
            
            if enabled:
                if shasattr(obj, 'fgField'):
                    myObjs.append(obj)
                if shasattr(obj, 'fieldsetFields'):
                    myObjs += obj.fieldsetFields(objTypes, includeFSMarkers)

        return myObjs


    security.declarePrivate('findFieldObjectByName')
    def findFieldObjectByName(self, name):
        """ Find a form field object by name,
            searching fieldsets if necessary.
            This is used by fieldsBase vocabulary fields
            to find the form field instance associated
            with a field.
        """
       
        for obj in self._getFieldObjects():
            if obj.__name__ == name:
               return obj
        return None        


    security.declarePrivate('cleanExpressionContext')
    def cleanExpressionContext(self, object=None, request=None):
        """ clean the expression context of references to object """

        # When a TALES expression is evaluated, a copy
        # of the expression context for the object is stored in the request
        # in case it's needed again.
        # The problem is that if we allow this to get stored for PFG's
        # overrides, it will not meet the needs of template code that
        # assumes a more complete context.
        #
        # This function's job is to clear the expression context cache for
        # the current object.
        #
        # Note that this doesn't need to be called if there's no danger
        # of the expression context being re-used in a template.

        if object is None:
            object = self
        
        if request:
            cache = request.get('_ec_cache', None)
            if cache:
                if cache.has_key( id(object) ):
                    del cache[ id(object) ]
        

    security.declareProtected(View, 'fgFields')
    def fgFields(self, request=None, displayOnly=False):
        """ generate fields on the fly; also primes request with
            defaults if request is passed.
            if displayOnly, label fields are excluded.
        """

        if request and self.getRawOnDisplayOverride():
            # call the tales expression, passing a custom context
            #self.getOnDisplayOverride(expression_context=getExprContext(self, self.aq_explicit))
            self.getOnDisplayOverride()
            self.cleanExpressionContext(request=request)

        myFields = []
        for obj in self._getFieldObjects(includeFSMarkers=not displayOnly):
            if IField.isImplementedBy(obj):
                # this is a field -- not a form field -- and may be
                # added directly to the field list.
                if not displayOnly:
                    myFields.append(obj)
            else:
                if request:
                    # prime the request
                    obj.fgPrimeDefaults(request)
                #if not (displayOnly and (obj.isLabel() or obj.isFileField()) ):
                if not (displayOnly and obj.isLabel()):
                    myFields.append(obj.fgField)

        return myFields

    security.declareProtected(View, 'fgvalidate')
    def fgvalidate(self, REQUEST=None, errors=None, data=None, metadata=None):
        """Validates the field data from the request.
        """

        _marker = []
        if errors is None:
            errors = {}
        if errors:
            return errors

        # Get all the form fields. Exclude actual IField fields.
        fields = [fo for fo in self._getFieldObjects() if not IField.isImplementedBy(fo)]

        for obj in fields:
            field = obj.fgField

            result = field.widget.process_form(self, field, REQUEST.form, empty_marker=_marker)

            if result is None or result is _marker:
                #XXX Make this smarter
                value = ''
            else:
                value = result[0]

            # workaround what I consider a Zope marshalling error: the production
            # of lists like ['one', ''] and [''] for list fields.
            # no need to worry about polymorphism here, as this is a very particular
            # case.        
            if isinstance(value, type([])) and len(value) and \
              (type(value[-1]) in StringTypes) and (len(value[-1]) == 0):
                value.pop()
                
            # eliminate trailing white space in string types.
            if safe_hasattr(value, 'rstrip'):
                newvalue = value.rstrip()
                if newvalue != value:
                    value = newvalue
                    # since strings are immutable, we have to manually store it back to the request
                    if safe_hasattr(REQUEST, 'form'):
                        REQUEST.form[obj.getFieldFormName()] = value                        

            # Archetypes field validation
            res = field.validate(instance=self, value=value, errors=errors, REQUEST=REQUEST)

            if not res:
                # give the field itself an opportunity to validate.
                res = obj.specialValidator(value, field, REQUEST, errors)

            if res:
                errors[field.getName()] = validationMessages.cleanupMessage(res, self.REQUEST, self)
            elif shasattr(obj, 'getFgTValidator') and obj.getRawFgTValidator():
                # process the override validator TALES expression

                # create a context for expression evaluation
                context = getExprContext(self, obj)
                
                # put this field's input (from request) into the context globals
                # as 'value'
                context.setGlobal('value', REQUEST.form.get(obj.getFieldFormName(), None))
                
                # call the tales expression, passing our custom context
                cerr = obj.getFgTValidator(expression_context=context)
                if cerr:
                    errors[field.getName()] = cerr

        if not errors:
            if self.getRawAfterValidationOverride():
                # evaluate the override.
                # In case we end up traversing to a template,
                # we need to make sure we don't clobber
                # the expression context.
                self.getAfterValidationOverride()
                self.cleanExpressionContext(request=self.REQUEST)

            adapters = self.getRawActionAdapter()
            for adapter in adapters:
                actionAdapter = getattr(self.aq_explicit, adapter, None)
                if actionAdapter is None:
                    logger.warn("Designated action adapter '%s' is missing; ignored." % adapter)
                else:
                    # Now, see if we should execute it.
                    # Check to see if execCondition exists and has contents
                    if safe_hasattr(actionAdapter, 'execCondition') and \
                      len(actionAdapter.getRawExecCondition()):
                        # evaluate the execCondition.
                        # create a context for expression evaluation
                        context = getExprContext(self, actionAdapter)
                        doit = actionAdapter.getExecCondition(expression_context=context)
                    else:
                        # no reason not to go ahead
                        doit = True

                    if doit:
                        result = actionAdapter.onSuccess(fields, REQUEST=REQUEST)
                        if type(result) is type({}) and len(result):
                            # return the dict, which hopefully uses
                            # field ids or FORM_ERROR_MARKER for keys
                            return result

        return errors


    security.declareProtected(View, 'fgGetSuccessAction')
    def fgGetSuccessAction(self):
        """
             Returns string id of success action template or script.
             Controller will traverse to this on successful validation.
         """

        if safe_hasattr(self, 'thanksPageOverride'):
            s = self.getThanksPageOverride()
            if s:
                return s

        s = getattr(self, 'thanksPage', None)
        if s:
            obj = getattr(self, s, None)
            if obj:
                return 'traverse_to:string:%s' % obj.getId()
            
        return 'traverse_to:string:fg_result_view'


    # security.declareProtected(ModifyPortalContent, 'getRawActionAdapter')
    def getRawActionAdapter(self):
        """ Returns selected action adapters as tuple """

        # this method translates a string actionAdapter
        # attribute from a previous version into a tuple        
        try:
            self.actionAdapter + ''
            if self.actionAdapter:
                return (self.actionAdapter,)
            else:
                return ()
        except:
            return self.actionAdapter


    security.declareProtected(ModifyPortalContent, 'actionAdaptersDL')
    def actionAdaptersDL(self):
        """ returns Display List (id, title) tuples of contained adapters """

        # an adapter provides IPloneFormGenActionAdapter
        allAdapters = [(obj.getId(), obj.title) for obj in self.objectValues() if IPloneFormGenActionAdapter in providedBy(obj)]

        if allAdapters:
            return DisplayList( allAdapters )

        return DisplayList()            
#        else:
#            if HAS_PLONE25:
#                return DisplayList(
#                    [('', _(u'vocabulary_none_text', u'None')),]
#                    )
#            else:
#                return DisplayList(
#                    [('', self.translate( msgid='vocabulary_none_text', domain='ploneformgen', default='None')),]
#                    )


    security.declareProtected(ModifyPortalContent, 'addActionAdapter')
    def addActionAdapter(self, id):
        """ activate action adapter with id == id """
        
        aa = list(self.getRawActionAdapter())
        if id not in aa:
            aa.append(id.decode(self.getCharset()))
        self.actionAdapter = aa
        

    security.declareProtected(ModifyPortalContent, 'fgFieldsDisplayList')
    def fgFieldsDisplayList(self, withNone=False, noneValue='', objTypes=None):
        """ returns display list of fields """

        myFields = []
        if withNone:
            if HAS_PLONE25:
                myFields.append( (noneValue, _(u'vocabulary_none_text', u'None')) )
            else:
                myFields.append( (noneValue, self.translate( msgid='vocabulary_none_text', domain='ploneformgen', default='None')) )

        for obj in self._getFieldObjects(objTypes):
            if isinstance(obj.title, unicode):
                myFields.append( (obj.getId(), obj.title) )
            else:
                myFields.append( (obj.getId(), obj.title.decode(self.getCharset())) )

        return DisplayList( myFields )


    security.declareProtected(ModifyPortalContent, 'thanksPageVocabulary')
    def thanksPageVocabulary(self):
        """ returns a DisplayList of contained page-ish documents """

        propsTool = getToolByName(self, 'portal_properties')
        siteProperties = getattr(propsTool, 'site_properties')
        defaultPageTypes = siteProperties.getProperty('default_page_types')

        if HAS_PLONE25:
            tpages = [('', _(u'vocabulary_none_text', u'None')),]
        else:
            tpages = [('', self.translate( msgid='vocabulary_none_text', domain='ploneformgen', default='None')),]

        for obj in self.objectValues():
            if IPloneFormGenThanksPage in providedBy(obj) or \
              getattr(obj.aq_explicit, 'portal_type', 'none') in defaultPageTypes:
                tpages.append( (obj.getId(), obj.title) )
                
        return DisplayList( tpages )


    ###
    # A few widgets (TextArea and RichWidget in particular) call the content
    # object rather than the field for this method. IMHO, this is unnecessary,
    # and should be fixed in the Widget. Meanwhile, this hack ...
    #
    security.declareProtected(View, 'isBinary')
    def isBinary(self, key):
        """Return whether a field contains binary data.
        """

        try:
            res = BaseObject.isBinary(self, key)
        except (TypeError, AttributeError):
            res = 0
        return res


    # The Archetypes file.pt macro tries -- for reasons I don't
    # really understand -- to call the field's accessor.
    # So, let's supply this as a harmless one when necessary.
    def nullAccessor(self, **kwargs):
        return None


    # Don't show the display options; doesn't make sense
    # when there are no real options
    def canSetDefaultPage(self):
        return False


    security.declarePrivate('_pfFixup')
    def _pfFixup(self, obj):
    
        # the creation of contained objects in initializeArchetypes
        # leaves catalog orphans for the portal_factory objects.
        #
        # this solves the problem by removing them from the
        # portal_catalog and uid_catalog,
        # hopefully not causing other problems in the process.

        if 'portal_factory' in obj.getPhysicalPath():
            # remove from portal_catalog
            obj.unindexObject()

            # remove from uid_catalog
            uid_catalog = getToolByName(self, 'uid_catalog', None)
            uid_catalog.uncatalog_object( getRelURL(self, obj.getPhysicalPath()) )

        else:
            obj.reindexObject()
        

    def initializeArchetype(self, **kwargs):
        """ Create sample content that may help folks
            figure out how this gadget works.
        """

        ATFolder.initializeArchetype(self, **kwargs)

        if HAS_PLONE25:
            self.setSubmitLabel(zope.i18n.translate(_(u'pfg_formfolder_submit', u'Submit'), context=self.REQUEST))
            self.setResetLabel(zope.i18n.translate(_(u'pfg_formfolder_reset', u'Reset'), context=self.REQUEST))
        else:
            self.setSubmitLabel(self.translate(
                                  msgid='pfg_formfolder_submit',
                                  domain='ploneformgen',
                                  default='Submit'))
            self.setResetLabel(self.translate(
                                  msgid='pfg_formfolder_reset',
                                  domain='ploneformgen',
                                  default='Reset'))

        oids = self.objectIds()

        haveMailer = False
        if 'mailer' not in oids:
            # create a mail action
            try:
                self.invokeFactory('FormMailerAdapter','mailer')
                mailer = self['mailer']

                if HAS_PLONE25:
                    mailer.setTitle(zope.i18n.translate(_(u'pfg_mailer_title', u'Mailer'), context=self.REQUEST))
                    mailer.setDescription(zope.i18n.translate(_(u'pfg_mailer_description', u'E-Mails Form Input'), context=self.REQUEST))
                else:
                    mailer.setTitle(self.utranslate(
                            msgid='pfg_mailer_title',
                            domain='ploneformgen',
                            default='Mailer'))
                    mailer.setDescription(self.utranslate(
                            msgid='pfg_mailer_description',
                            domain='ploneformgen',
                            default='E-Mails From Input'))

                self._pfFixup(mailer)

                self.actionAdapter = ('mailer',)
                haveMailer = True
            except Unauthorized:
                logger.warn('User not authorized to create mail adapters. Form Folder created with no action adapter.')

        if 'replyto' not in oids:
            # create a replyto field
            self.invokeFactory('FormStringField','replyto')
            obj = self['replyto']
            obj.fgField.__name__ = 'replyto'

            if HAS_PLONE25:
                obj.setTitle(zope.i18n.translate(_(u'pfg_replytofield_title', u'Your E-Mail Address'), context=self.REQUEST))
            else:
                obj.setTitle(self.translate(
                                  msgid='pfg_replytofield_title',
                                  domain='ploneformgen',
                                  default='Your E-Mail Address'))

            obj.fgField.required = True
            obj.setFgStringValidator('isEmail')
            obj.setFgTDefault('here/memberEmail')
            obj.setFgDefault('dynamically overridden')

            self._pfFixup(obj)

            if haveMailer:
                mailer.replyto_field = 'replyto'
        
        if 'topic' not in oids:
            # create a subject field
            self.invokeFactory('FormStringField','topic')
            obj = self['topic']
            obj.fgField.__name__ = 'topic'

            if HAS_PLONE25:
                obj.setTitle(zope.i18n.translate(_(u'pfg_topicfield_title', u'Subject'), context=self.REQUEST))
            else:
                obj.setTitle(self.translate(
                                  msgid='pfg_topicfield_title',
                                  domain='ploneformgen',
                                  default='Subject'))

            obj.fgField.required = True

            self._pfFixup(obj)

            if haveMailer:
                mailer.subject_field = 'topic'
        
        if 'comments' not in oids:
            # create a comments field
            self.invokeFactory('FormTextField','comments')
            obj = self['comments']
            obj.fgField.__name__ = 'comments'

            if HAS_PLONE25:
                obj.setTitle(zope.i18n.translate(_(u'pfg_commentsfield_title', u'Comments'), context=self.REQUEST))
            else:
                obj.setTitle(self.translate(
                                msgid='pfg_commentsfield_title',
                                domain='ploneformgen',
                                default='Comments'))

            obj.fgField.required = True

            self._pfFixup(obj)

        
        if 'thank-you' not in oids:
            # create a thanks page
            self.invokeFactory('FormThanksPage','thank-you')
            obj = self['thank-you']

            if HAS_PLONE25:
                obj.setTitle(zope.i18n.translate(_(u'pfg_thankyou_title', u'Thank You'), context=self.REQUEST))
                obj.setDescription(zope.i18n.translate(_(u'pfg_thankyou_description', u'Thanks for your input.'), context=self.REQUEST))
            else:
                obj.setTitle(self.translate(
                                    msgid='pfg_thankyou_title',
                                    domain='ploneformgen',
                                    default='Thank You'))
                obj.setDescription(self.translate(
                                    msgid='pfg_thankyou_description',
                                    domain='ploneformgen',
                                    default='Thanks for your input.'))

            self._pfFixup(obj)

            self.thanksPage = 'thank-you'
                    

    security.declareProtected(View, 'memberFullName')
    def memberFullName(self):
        """ convenience method meant for use in default overrides.
            returns full name of authenticated user, if available,
            empty string otherwise.
        """
    
        pm = getToolByName(self, 'portal_membership')
        member = pm.getAuthenticatedMember()

        return member.getProperty('fullname', '')


    security.declareProtected(View, 'memberEmail')
    def memberEmail(self):
        """ convenience method meant for use in default overrides.
            returns e-mail address of authenticated user, if available,
            empty string otherwise.
        """
    
        pm = getToolByName(self, 'portal_membership')
        member = pm.getAuthenticatedMember()
        
        return member.getProperty('email', '')


    security.declareProtected(View, 'memberId')
    def memberId(self):
        """ convenience method meant for use in default overrides.
            returns login id of authenticated user, if available,
            empty string otherwise.
        """
    
        pm = getToolByName(self, 'portal_membership')
        if pm.isAnonymousUser():
            return ''

        member = pm.getAuthenticatedMember()
        return member.id


    # security is inherited
    def checkIdAvailable(self, id):
        """ Expands on ATFolder by checking for ids known to cause problems.
            This includes ids of objects in all fieldsets.
        """
        
        result = ATFolder.checkIdAvailable(self, id)
        if result:
            result = id not in BAD_IDS
            if result:
                # check the fieldsets
                fieldsets = self.objectValues('FieldsetFolder')
                for fs in fieldsets:
                    if id in fs.objectIds():
                        return False
                
        return result
        

    security.declareProtected(View, 'formFolderObject')
    def formFolderObject(self):
        """ Find form folder by acquisition """
        
        return self

 
    security.declareProtected(ModifyPortalContent, 'setFormPrologue')
    def setFormPrologue(self, value, **kw):
        """ Set formPrologue """
        
        # workaround a Kupu oddity: saving '<p>&nbsp;</p>' for
        # and empty input
        if value.strip() == '<p>&nbsp;</p>':
            self.formPrologue = ''
        else:
            self.formPrologue = value


    security.declareProtected(ModifyPortalContent, 'setFormPrologue')
    def setFormEpilogue(self, value, **kw):
        """ Set formEpilogue """
        
        # workaround a Kupu oddity: saving '<p>&nbsp;</p>' for
        # and empty input
        if value.strip() == '<p>&nbsp;</p>':
            self.formEpilogue = ''
        else:
            self.formEpilogue = value


#    security.declareProtected(ModifyPortalContent, 'myi18n')
#    def myi18n(self):
#        """ return i18n declarations from widgets """
#
#        boilerplate = """\n#. %s\n#. Default: "%s"\nmsgid "%s"\nmsgstr ""\n"""
#
#        res = ''
#
#        from Products.PloneFormGen.content.fields import *
#        from Products.PloneFormGen.content.formMailerAdapter import *
#        from Products.PloneFormGen.content.saveDataAdapter import *
#        from Products.PloneFormGen.content.thanksPage import *
#        
#        
#        klasses = (
#            FormFolder,
#            FGStringField,
#            FGPasswordField,
#            FGIntegerField,
#            FGFixedPointField,
#            FGBooleanField,
#            FGDateField,
#            FGLabelField,
#            FGLinesField,
#            FGSelectionField,
#            FGMultiSelectField,
#            FGTextField,
#            FGRichTextField,
#            FGFileField,
#            FormMailerAdapter,
#            FormSaveDataAdapter,
#            FormThanksPage,
#            )
#
#        done = {}
#
#        for myclass in klasses:        
#            myname = myclass.archetype_name
#            myschema = myclass.schema
#            
#            # res = res + "\n### %s ###\n" % myname
#            
#            for aschema in myschema.getSchemataNames():
#                for field in myschema.getSchemataFields(aschema):
#                    widget = field.widget
#                    domain = getattr(widget, 'i18n_domain', None)
#                    
#                    if domain == 'ploneformgen':
#                    
#
#                        id = getattr(widget, 'label_msgid', '***NO label_msgid***')
#                        val = widget.label
#                        if done.get(id) != val:
#                            msg = boilerplate % (myname, val, id)
#                            res = "%s%s\n" % (res, msg)
#                            done[id] = val
#                        #else:
#                        #    res = res + "\navoided repeating %s for %s" % (id, myname)
#                        
#                        desc = widget.description
#                        if desc:
#                            desid = getattr(widget, 'description_msgid', '***NO description_msgid***')
#                            if done.get(desid) != desc:
#                                mdesc = "\n#. Default: ".join( [ln.strip() for ln in desc.split('\n')] )
#                                msg = boilerplate % (myname, mdesc, desid)
#                                done[desid] = desc
#                                res = "%s%s\n" % (res, msg)
#                            #else:
#                            #    res = res + "\navoided repeating %s for %s" % (desid, myname)
#
#                    elif domain not in ('plone', 'atcontenttypes',):
#                        res = "%s\n***Unexpected domain for %s:%s: %s\n" % (res, myname, widget.label, domain)
#            
#        return res


registerATCT(FormFolder, PROJECTNAME)



#
# Likert Field related tests
#

import os, sys, email

from ZPublisher.HTTPRequest import record

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PloneFormGen.tests import pfgtc

from Products.CMFCore.utils import getToolByName

class FakeRequest(dict):

    def __init__(self, **kwargs):
        self.form = kwargs

class TestLikertField(pfgtc.PloneFormGenTestCase):
    """ tests that mostly concern the Likert Field.
        field instantiation is tested in testSetup,
        so we don't bother with it here.
    """

    def afterSetUp(self):
        # build a form folder, add a Likert Field and
        # a saver.
        pfgtc.PloneFormGenTestCase.afterSetUp(self)
        self.folder.invokeFactory('FormFolder', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.ff1.invokeFactory('FormLikertField', 'lf')
        self.lf = getattr(self.ff1, 'lf')
        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')
        self.saver = getattr(self.ff1, 'saver')
        self.ff1.setActionAdapter( ('saver',) )

    def test_Setup(self):
        """ Test field setup """
        lf = self.lf
        # check default question and answer set
        self.assertEqual(len(lf.likertQuestions), 2)
        self.assertEqual(len(lf.likertAnswers), 5)
        self.assertEqual(len(lf.fgField.questionSet), 2)
        self.assertEqual(len(lf.fgField.answerSet), 5)
            

    def test_Validate(self):
        """ Test required field validation """

        # try it with no input, unrequired
        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )
        self.assertEqual(self.saver.itemsSaved(), 1)
        self.assertEqual(self.saver.getSavedFormInput()[0][3], '')

        # try it with no input, required
        self.lf.setRequired(True)
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {'lf': u'pfg_allRequired'} )

        # try it with input, required
        request = FakeRequest(topic = 'test subject', 
                              replyto='test@test.org', 
                              comments='test comments',
                              lf={'1':'2','2':'3'})
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )
        self.assertEqual(self.saver.itemsSaved(), 2)
        self.assertEqual(self.saver.getSavedFormInput()[1][3], "{'1': '2', '2': '3'}")
    
    def test_likert_html_output(self):
        """Failing test for #225, AttributeError: __len__" in Plone 2.5.5"""
        rating_req_val = record()
        rating_req_val.__dict__ = {'1':'2','2':'3'}
        request = FakeRequest(topic = 'test subject', replyto='test@test.org', 
                              comments='test comments', lf=rating_req_val)
        self.failUnless("1: 2, 2: 3" in self.ff1.lf.htmlValue(request))
    


if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestLikertField))
    return suite



# Integration tests specific to save-data adapter.
#

import os, sys, email

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PloneFormGen.tests import pfgtc

from Products.CMFCore.utils import getToolByName

# dummy class
class cd:
    pass

class FakeRequest(dict):
    
    def __init__(self, **kwargs):
        self.form = kwargs

class TestFunctions(pfgtc.PloneFormGenTestCase):
    """ test save data adapter """
    
    def afterSetUp(self):
        pfgtc.PloneFormGenTestCase.afterSetUp(self)
        self.folder.invokeFactory('FormFolder', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')

    def testSaver(self):
        """ test save data adapter action """

        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')

        self.failUnless('saver' in self.ff1.objectIds())
        saver = self.ff1.saver
        
        self.ff1.setActionAdapter( ('saver',) )
        self.assertEqual(self.ff1.actionAdapter, ('saver',))

        # print "|%s|" % saver.SavedFormInput
        self.assertEqual(saver.itemsSaved(), 0)

        res = saver.getSavedFormInputForEdit()
        self.assertEqual(res, '')

        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )
        
        self.assertEqual(saver.itemsSaved(), 1)

        res = saver.getSavedFormInputForEdit()
        self.assertEqual(res.strip(), 'test@test.org,test subject,test comments')


    def testSaverSavedFormInput(self):
        """ test save data adapter action and direct access to SavedFormInput """

        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')

        self.failUnless('saver' in self.ff1.objectIds())
        saver = self.ff1.saver
        
        self.ff1.setActionAdapter( ('saver',) )

        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )
        
        self.assertEqual(saver.itemsSaved(), 1)
        row = iter(saver.getSavedFormInput()).next()
        self.assertEqual(len(row), 3)
        
        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )
        self.assertEqual(saver.itemsSaved(), 2)

        self.ff1.saver.clearSavedFormInput()
        self.assertEqual(saver.itemsSaved(), 0)


    def testSetSavedFormInput(self):
        """ test setSavedFormInput functionality """

        # set up saver        
        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')
        self.failUnless('saver' in self.ff1.objectIds())
        saver = self.ff1.saver
        
        # save a row
        saver.setSavedFormInput('one,two,three')
        self.assertEqual(saver.itemsSaved(), 1)
        self.assertEqual(saver._inputStorage[0], ['one', 'two', 'three'])

        # save a couple of \n-delimited rows - \n eol
        saver.setSavedFormInput('one,two,three\nfour,five,six')
        self.assertEqual(saver.itemsSaved(), 2)
        self.assertEqual(saver._inputStorage[0], ['one', 'two', 'three'])
        self.assertEqual(saver._inputStorage[1], ['four', 'five', 'six'])

        # save a couple of \n-delimited rows -- \r\n eol
        saver.setSavedFormInput('one,two,three\r\nfour,five,six')
        self.assertEqual(saver.itemsSaved(), 2)

        # save a couple of \n-delimited rows -- \n\n double eol
        saver.setSavedFormInput('one,two,three\n\nfour,five,six')
        self.assertEqual(saver.itemsSaved(), 2)

        # save empty string
        saver.setSavedFormInput('')
        self.assertEqual(saver.itemsSaved(), 0)

        # save empty list
        saver.setSavedFormInput(tuple())
        self.assertEqual(saver.itemsSaved(), 0)

    def testEditSavedFormInput(self):
        """ test manage_saveData functionality """

        # set up saver        
        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')
        self.failUnless('saver' in self.ff1.objectIds())
        saver = self.ff1.saver

        # save a row
        saver.setSavedFormInput('one,two,three')
        self.assertEqual(saver.itemsSaved(), 1)
        self.assertEqual(saver._inputStorage[0], ['one', 'two', 'three'])
        
        data = cd()
        setattr(data, 'item-0', 'four')
        setattr(data, 'item-1', 'five')
        setattr(data, 'item-2', 'six')

        saver.manage_saveData(1, data)
        self.assertEqual(saver.itemsSaved(), 1)
        self.assertEqual(saver._inputStorage[0], ['four', 'five', 'six'])

    def testDeleteSavedFormInput(self):
        """ test manage_deleteData functionality """

        # set up saver        
        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')
        self.failUnless('saver' in self.ff1.objectIds())
        saver = self.ff1.saver

        # save a few rows
        saver._addDataRow( ['one','two','three'] )
        saver._addDataRow( ['four','five','six'] )
        saver._addDataRow( ['seven','eight','nine'] )
        self.assertEqual(saver.itemsSaved(), 3)
        
        saver.manage_deleteData(2)
        self.assertEqual(saver.itemsSaved(), 2)
        self.assertEqual(saver._inputStorage[0], ['one', 'two', 'three'])
        self.assertEqual(saver._inputStorage[1], ['seven', 'eight', 'nine'])
    

    def testSaverInputAsDictionaries(self):
        """ test save data adapter's InputAsDictionaries """
        
        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')

        self.failUnless('saver' in self.ff1.objectIds())
        saver = self.ff1.saver
        
        self.ff1.setActionAdapter( ('saver',) )

        self.assertEqual(saver.inputAsDictionaries, saver.InputAsDictionaries)

        self.assertEqual(saver.itemsSaved(), 0)

        request = FakeRequest(topic = 'test subject', replyto='test@test.org', comments='test comments')
        errors = self.ff1.fgvalidate(REQUEST=request)
        self.assertEqual( errors, {} )

        self.assertEqual(saver.itemsSaved(), 1)
        
        iad = saver.InputAsDictionaries()        
        row = iter(iad).next()
        self.assertEqual(len(row), 3)
        self.assertEqual(row['topic'], 'test subject')
        


    def testSaverColumnNames(self):
        """ test save data adapter's getColumnNames function """

        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')

        self.failUnless('saver' in self.ff1.objectIds())
        saver = self.ff1.saver
        
        self.ff1.setActionAdapter( ('saver',) )
        
        cn = saver.getColumnNames()
        self.failUnless( len(cn) == 3 )
        self.failUnless( cn[0] == 'replyto')
        self.failUnless( cn[1] == 'topic')
        self.failUnless( cn[2] == 'comments')
        
        # Add an extra column
        saver.ExtraData = ('dt',)
        cn = saver.getColumnNames()
        self.failUnless( len(cn) == 4 )
        self.failUnless( cn[3] == 'dt' )

        # add a label field -- should not show up in column names
        self.ff1.invokeFactory('FormLabelField', 'alabel')
        cn = saver.getColumnNames()
        self.failUnless( len(cn) == 4 )

        # add a form field -- should show up in column names before 'dt'        
        self.ff1.invokeFactory('FormFileField', 'afile')
        cn = saver.getColumnNames()
        self.failUnless( len(cn) == 5 )
        self.failUnless( cn[3] == 'afile')
        self.failUnless( cn[4] == 'dt')
        
        
    def testSaverColumnTitles(self):
        """ test save data adapter's getColumnTitles function """

        self.ff1.invokeFactory('FormSaveDataAdapter', 'saver')

        self.failUnless('saver' in self.ff1.objectIds())
        saver = self.ff1.saver
        
        self.ff1.setActionAdapter( ('saver',) )
        
        cn = saver.getColumnTitles()
        self.failUnless( len(cn) == 3 )
        self.failUnless( cn[0] == 'Your E-Mail Address')
        self.failUnless( cn[1] == 'Subject')
        self.failUnless( cn[2] == 'Comments')

        # Add an extra column
        saver.ExtraData = ('dt',)
        cn = saver.getColumnTitles()
        self.failUnless( len(cn) == 4 )
        self.failUnless( cn[3] == 'Posting Date/Time' )



if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestFunctions))
    return suite



# coding=utf-8
#
# Integeration tests specific to the mailer
#

import os, sys, email

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PloneFormGen.tests import pfgtc

from Products.CMFCore.utils import getToolByName

class TestFunctions(pfgtc.PloneFormGenTestCase):
    """ test ya_gpg.py """
    
    def dummy_send( self, mfrom, mto, messageText ):
        self.mfrom = mfrom
        self.mto = mto
        self.messageText = messageText


    def afterSetUp(self):
        pfgtc.PloneFormGenTestCase.afterSetUp(self)
        self.folder.invokeFactory('FormFolder', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
        self.mailhost = self.folder.MailHost
        self.mailhost._send = self.dummy_send        
        self.ff1.mailer.setRecipient_email('mdummy@address.com')

    
    def LoadRequestForm(self, **kwargs):
        form = self.app.REQUEST.form
        form.clear()
        for key in kwargs.keys():
            form[key] = kwargs[key]
        return self.app.REQUEST


    def test_DummyMailer(self):
        """ sanity check; make sure dummy mailer works as expected """

        self.mailhost.send('messageText', mto='dummy@address.com', mfrom='dummy1@address.com')
        self.failUnless( self.messageText.endswith('messageText') )
        self.assertEqual(self.mto, ['dummy@address.com'])
        self.failUnless( self.messageText.find('To: dummy@address.com') > 0 )
        self.assertEqual(self.mfrom,'dummy1@address.com')
        self.failUnless( self.messageText.find('From: dummy1@address.com') > 0 )
        # print "|%s" % self.messageText


    def test_Mailer(self):
        """ Test mailer with dummy_send """
    
        mailer = self.ff1.mailer
        
        fields = self.ff1._getFieldObjects()
        
        request = self.LoadRequestForm(topic = 'test subject', comments='test comments')
        
        mailer.onSuccess(fields, request)
        
        self.failUnless( self.messageText.find('To: <mdummy@address.com>') > 0 )
        self.failUnless( self.messageText.find('Subject: =?utf-8?q?test_subject?=') > 0 )
        msg = email.message_from_string(self.messageText)
        self.failUnless( msg.get_payload(decode=True).find('test comments') > 0 )


    def test_MailerLongSubject(self):
        """ Test mailer with subject line > 76 chars (Tracker #84) """

        long_subject = "Now is the time for all good persons to come to the aid of the quick brown fox."
    
        mailer = self.ff1.mailer        
        fields = self.ff1._getFieldObjects()        
        request = self.LoadRequestForm(topic = long_subject)        
        mailer.onSuccess(fields, request)
        
        msg = email.message_from_string(self.messageText)
        encoded_subject_header = msg['subject']
        decoded_header = email.Header.decode_header(encoded_subject_header)[0][0]
        
        self.assertEqual( decoded_header, long_subject )


    def test_MailerOverrides(self):
        """ Test mailer override functions """
    
        mailer = self.ff1.mailer
        mailer.setSubjectOverride("python: '%s and %s' % ('eggs', 'spam')")
        mailer.setSenderOverride("string: spam@eggs.com")
        mailer.setRecipientOverride("string: eggs@spam.com")
        
        fields = self.ff1._getFieldObjects()
        
        request = self.LoadRequestForm(topic = 'test subject')
        
        mailer.onSuccess(fields, request)
        
        # print "|%s" % self.messageText
        self.failUnless( self.messageText.find('Subject: =?utf-8?q?eggs_and_spam?=') > 0 )
        self.failUnless( self.messageText.find('From: spam@eggs.com') > 0 )
        self.failUnless( self.messageText.find('To: <eggs@spam.com>') > 0 )


    def testMultiRecipientOverrideByString(self):
        """ try multiple recipients in recipient override """
        
        mailer = self.ff1.mailer
        mailer.setRecipientOverride("string: eggs@spam.com, spam@spam.com")
        
        fields = self.ff1._getFieldObjects()
        
        request = self.LoadRequestForm(topic = 'test subject')
        
        mailer.onSuccess(fields, request)
        
        self.failUnless( self.messageText.find('To: <eggs@spam.com>, <spam@spam.com>') > 0 )
        


    def testMultiRecipientOverrideByTuple(self):
        """ try multiple recipients in recipient override """
        
        mailer = self.ff1.mailer
        mailer.setRecipientOverride("python: ('eggs@spam.com', 'spam.spam.com')")
        
        fields = self.ff1._getFieldObjects()
        
        request = self.LoadRequestForm(topic = 'test subject')
        
        mailer.onSuccess(fields, request)
        
        # print "|%s" % self.messageText
        self.failUnless( self.messageText.find('To: <eggs@spam.com>, <spam.spam.com>') > 0 )
        


    def testRecipientFromRequest(self):
        """ try recipient from designated field  """
        
        mailer = self.ff1.mailer
        mailer.setTo_field("selField")
        
        fields = self.ff1._getFieldObjects()
        
        request = self.LoadRequestForm(topic = 'test subject', selField = 'eggs@spamandeggs.com')
        
        mailer.onSuccess(fields, request)
        
        # print "|%s" % self.messageText
        self.failUnless( self.messageText.find('To: <eggs@spamandeggs.com>') > 0 )

        request = self.LoadRequestForm(topic = 'test subject', selField = ['eggs@spam.com', 'spam@spam.com'])
        
        mailer.onSuccess(fields, request)
        
        # print "|%s" % self.messageText
        self.failUnless( self.messageText.find('To: <eggs@spam.com>, <spam@spam.com>') > 0 )
        



    def test_ExecConditions(self):
        """ Test mailer with various exec conditions """

        # if an action adapter's execCondition is filled in and evaluates false,
        # the action adapter should not fire.

        mailer = self.ff1.mailer
        request = self.LoadRequestForm(topic = 'test subject', replyto='test@test.org', comments='test comments')

        self.messageText = ''
        mailer.setExecCondition('python: False')
        self.assertEqual( self.ff1.fgvalidate(REQUEST=request), {} )
        self.failUnless( len(self.messageText) == 0 )
        
        self.messageText = ''
        mailer.setExecCondition('python: True')
        self.assertEqual( self.ff1.fgvalidate(REQUEST=request), {} )
        self.failUnless( len(self.messageText) > 0 )

        self.messageText = ''
        mailer.setExecCondition('python: 1==0')
        self.assertEqual( self.ff1.fgvalidate(REQUEST=request), {} )
        self.failUnless( len(self.messageText) == 0 )

        # make sure an empty execCondition causes the action to fire
        self.messageText = ''
        mailer.setExecCondition('')
        self.assertEqual( self.ff1.fgvalidate(REQUEST=request), {} )
        self.failUnless( len(self.messageText) > 0 )



if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestFunctions))
    return suite



from zope.interface import Interface

class IPloneFormGenForm(Interface):
    """fgForm marker interface
    """
    
    def fgFields(request=None):
        """ generate fields on the fly; also primes request with defaults if request is passed """
    
    def fgvalidate(REQUEST=None, errors=None, data=None, metadata=None):
        """Validates the field data from the request.
        """




from Products.PloneFormGen import HAS_PLONE30
from Products.PloneFormGen.config import *

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ManagePortal

from Products.CMFPlone.utils import safe_hasattr

from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin

from StringIO import StringIO


allTypes = ('FormFolder',) + fieldTypes + adapterTypes + thanksTypes + fieldsetTypes

# Configlets to be added to control panels or removed from them
configlets = (
        { 'id'         : 'PloneFormGen'
        , 'name'       : 'PloneFormGen'
        , 'action'     : 'string:${portal_url}/prefs_pfg_permits'
        , 'condition'  : ''
        , 'category'   : 'Products'
        , 'visible'    : 1
        , 'appId'      : PROJECTNAME
        , 'permission' : ManagePortal
        , 'imageUrl'   : 'Form.gif'
        },
    )


def install(self):
    out = StringIO()

    print >> out, "Installing PloneFormGen"
    # PFG was using this generic setup profile due to an Archetypes 1.5b5
    # bug that was fixed in 1.5rc3.
    # if HAS_PLONE30:
    #     # Archetypes 1.5.0b5 (included with Plone 3.0b3) has a bug which prevents
    #     # installTypes from working unless class name == portal_type.
    #     # Work around it by using portal_setup to handle this part of the install.
    #     #
    #     setup_tool = getToolByName(self, 'portal_setup')
    #     old_context = setup_tool.getImportContextID()
    #     setup_tool.setImportContext('profile-Products.PloneFormGen:typesonly')
    #     setup_tool.runAllImportSteps()
    #     setup_tool.setImportContext(old_context)
    #     print >> out, "Installed types and added to portal_factory via portal_setup"
    # else:
	# Install types
    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                classes,
                PROJECTNAME)
    
    # for reinstalls: avoid clobbering contained types set up by 3rd-party products
    if hasattr(self.aq_explicit, '_v_pfg_old_allowed_types'):
        pt = getToolByName(self, 'portal_types')
        new_allowed_types = list(pt['FormFolder'].allowed_content_types)
        for t in self._v_pfg_old_allowed_types:
            if t not in new_allowed_types:
                new_allowed_types.append(t)
        pt['FormFolder'].allowed_content_types = new_allowed_types
        delattr(self, '_v_pfg_old_allowed_types')
    print >> out, "Installed types"
    
        # Enable portal_factory
    factory = getToolByName(self, 'portal_factory')
    types = factory.getFactoryTypes().keys()
    for f in allTypes:
        if f not in types:
            types.append(f)
    factory.manage_setPortalFactoryTypes(listOfTypeIds = types)
    print >> out, "Added all my types to portal_factory"

    if HAS_PLONE30:
        # hide properties/references tabs
        pt = getToolByName(self, 'portal_types')
        for typ in types:
            try:
                for act in pt[typ].listActions():
                    if act.id in ['metadata', 'references']:
                        act.visible = False
            except KeyError:
                # prevent breaking on edge case: portal_factories still lists a type that
                # is no longer present in portal_types
                pass

    ## Install skin
    install_subskin(self, out, GLOBALS)
    print >> out, "Installed skin"

    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')

    # Add the field, fieldset, thanks and adapter types to types_not_searched
    typesNotSearched = list(siteProperties.getProperty('types_not_searched'))
    for f in fieldTypes + adapterTypes + thanksTypes + fieldsetTypes:
        if f not in typesNotSearched:
            typesNotSearched.append(f)
    siteProperties.manage_changeProperties(types_not_searched = typesNotSearched)
    print >> out, "Added form fields & adapters to types_not_searched"

    # Add the field, fieldset, thanks and adapter types to types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in fieldTypes + adapterTypes + thanksTypes + fieldsetTypes:
        if f not in typesNotListed:
            typesNotListed.append(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Added form fields & adapters to metaTypesNotToList"

    # Set up the workflow for the field, fieldset, thanks and adapter types: there should be none!
    wft = getToolByName(self, 'portal_workflow')
    wft.setChainForPortalTypes(fieldTypes + adapterTypes + thanksTypes + fieldsetTypes, ())
    print >> out, "Set up empty field and adapter workflows."

    # Add to default_page_types
    defaultPageTypes = list(siteProperties.getProperty('default_page_types'))
    if 'FormFolder' not in defaultPageTypes:
        defaultPageTypes.append('FormFolder')
    siteProperties.manage_changeProperties(default_page_types = defaultPageTypes)
    print >> out, "Added FormFolder to default_page_types"

    # Add FormFolder to kupu's linkable types
    kupuTool = getToolByName(self, 'kupu_library_tool', None)
    if kupuTool is not None:
        linkable = list(kupuTool.getPortalTypesForResourceType('linkable'))
        if 'FormFolder' not in linkable:
            linkable.append('FormFolder')
        # See optilude's note in the RichDocument install re why this is so odd.
        kupuTool.updateResourceTypes(({'resource_type' : 'linkable',
                                       'old_type'      : 'linkable',
                                       'portal_types'  :  linkable},))
        print >> out, "Added FormFolder to kupu's linkable types"

    if not safe_hasattr(self, 'formgen_tool'):
        portalObject = getToolByName(self, 'portal_url').getPortalObject()
        addTool = portalObject.manage_addProduct['PloneFormGen'].manage_addTool
        addTool('PloneFormGen Tool')
    print >> out, "Added PloneFormGen Tool"

    
    # add property sheet in portal_properties
    # we'll use this to store site defaults
    ppTool = getToolByName(self, 'portal_properties')
    if not safe_hasattr(ppTool, PROPERTY_SHEET_NAME):
        ppTool.addPropertySheet(PROPERTY_SHEET_NAME, 'PloneFormGen properties')
        print >> out, "Added PloneFormGen properysheet"
    else:
        print >> out, "Using existing propertysheet"
    propSheet = getattr(ppTool, PROPERTY_SHEET_NAME)
    if not propSheet.hasProperty('permissions_used'):
        propSheet.manage_addProperty('permissions_used', pfgPermitList, 'lines')
    if not propSheet.hasProperty('mail_template'):
        propSheet.manage_addProperty('mail_template', DEFAULT_MAILTEMPLATE_BODY, 'text')    
    if not propSheet.hasProperty('mail_body_type'):
        propSheet.manage_addProperty('mail_body_type', 'html', 'string')    
    if not propSheet.hasProperty('mail_recipient_email'):
        propSheet.manage_addProperty('mail_recipient_email', '', 'string')    
    if not propSheet.hasProperty('mail_recipient_name'):
        propSheet.manage_addProperty('mail_recipient_name', '', 'string')
    if not propSheet.hasProperty('mail_cc_recipients'):
        propSheet.manage_addProperty('mail_cc_recipients', [], 'lines')
    if not propSheet.hasProperty('mail_bcc_recipients'):
        propSheet.manage_addProperty('mail_bcc_recipients', [], 'lines')
    if not propSheet.hasProperty('mail_xinfo_headers'):
        propSheet.manage_addProperty('mail_xinfo_headers', XINFO_DEFAULT, 'lines')
    if not propSheet.hasProperty('mail_add_headers'):
        propSheet.manage_addProperty('mail_add_headers', [], 'lines')
    

    # add the configlets to the portal control panel
    configTool = getToolByName(self, 'portal_controlpanel', None)
    productConfiglets = [co['id'] for co in configTool.enumConfiglets(group='Products')]
    if 'PloneFormGen' not in productConfiglets:
        for conf in configlets:
            try:
                configTool.registerConfiglet(**conf)
                out.write('Added configlet %s\n' % conf['id'])
            except:
                out.write('Configlet already configured\n')
    else:
        out.write('Unexpectedly found an existing configlet for PFG. Skipped configlet registration.')        


    if HAS_PLONE30:
        # register kss resource
        kssRegTool = getToolByName(self, 'portal_kss', None)
        # if our kss item is already in the registry, let's
        # yank it out so that we force the registry to update
        try:
            kssRegTool.manage_removeKineticStylesheet('ploneformgen.kss')
        except:
            pass
        kssRegTool.registerKineticStylesheet('ploneformgen.kss')
        out.write('Added ploneformgen.kss to kss registry\n')


    ##
    ## Print install info
    ##
    print >> out, "Successfully installed %s." % PROJECTNAME
    return out.getvalue()


def removeSkinLayer(self, layer):
    """ Remove a skin layer from all skinpaths """

    skinstool = getToolByName(self, 'portal_skins')
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName)
        path = [i.strip() for i in  path.split(',')]
        if layer in path:
            path.remove(layer)
            path = ','.join(path)
            skinstool.addSkinSelection(skinName, path)


def addSkinLayer(self, layer):
    """ Add a layer immediately after custom to all skinpaths """
    
    skinstool = getToolByName(self, 'portal_skins')
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName)
        path = [i.strip() for i in  path.split(',')]
        try:
            if layer not in path:
                path.insert(path.index('custom') +1, layer)
        except ValueError:
            if layer not in path:
                path.append(layer)
        path = ','.join(path)
        skinstool.addSkinSelection(skinName, path)


def uninstall_tool(self, out):
    try:
        self.manage_delObjects(['formgen_tool'])
    except:
        pass
    else:
        print >>out, "PloneFormGen tool removed"


def uninstall_configlet(self, out):
    # remove the configlets from the portal control panel
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            configTool.unregisterConfiglet(conf['id'])
            out.write('Removed configlet %s\n' % conf['id'])

def beforeUninstall(self, reinstall, product, cascade):
    # for reinstalls: store list of allowed contained types,
    # so we don't lose anything that a 3rd-party product added.
    # This is ugly, but I'm not sure where else to stash this,
    # since this is an external method and I can't seem to use
    # a global variable.
    pt = getToolByName(self, 'portal_types')
    try:
        self._v_pfg_old_allowed_types = pt['FormFolder'].allowed_content_types
    except KeyError:
        self._v_pfg_old_allowed_types = None

    return '', cascade

def uninstall(self):
    out = StringIO()

    uninstall_tool(self, out)
    
    uninstall_configlet(self, out)
    

    # remove FormFolder from kupu's linkable types
    kupuTool = getToolByName(self, 'kupu_library_tool')
    linkable = list(kupuTool.getPortalTypesForResourceType('linkable'))
    if 'FormFolder' in linkable:
        linkable.remove('FormFolder')
    kupuTool.updateResourceTypes(({'resource_type' : 'linkable',
                                   'old_type'      : 'linkable',
                                   'portal_types'  :  linkable},))
    print >> out, "Removed FormFolder from kupu's linkable types"


    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')

    # remove the field, thanks and adapter types from types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in fieldTypes + adapterTypes + thanksTypes, fieldsetTypes:
        if f in typesNotListed:
            typesNotListed.remove(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Removed form fields & adapters from metaTypesNotToList"

    # Remove the field, thanks and adapter types from types_not_searched
    typesNotSearched = list(siteProperties.getProperty('types_not_searched'))
    for f in fieldTypes + adapterTypes + thanksTypes + fieldsetTypes:
        if f in typesNotSearched:
            typesNotSearched.remove(f)
    siteProperties.manage_changeProperties(types_not_searched = typesNotSearched)
    print >> out, "Removed form fields & adapters from types_not_searched"

    # Remove from default_page_types
    defaultPageTypes = list(siteProperties.getProperty('default_page_types'))
    if 'FormFolder' in defaultPageTypes:
        defaultPageTypes.remove('FormFolder')
        siteProperties.manage_changeProperties(default_page_types = defaultPageTypes)
    print >> out, "Removed FormFolder from default_page_types"

    # Remove from use_folder_tabs 
    use_folder_tabs = list(siteProperties.getProperty('use_folder_tabs'))
    if 'FormFolder' in use_folder_tabs:
        use_folder_tabs.remove('FormFolder')
    if 'FieldsetFolder' in use_folder_tabs:
        use_folder_tabs.remove('FieldsetFolder')
    siteProperties.manage_changeProperties(use_folder_tabs = use_folder_tabs)
    print >> out, "Removed FormFolder and FieldsetFolder from use_folder_tabs"

    # Remove from typesLinkToFolderContentsInFC 
    typesLinkToFolderContentsInFC = list(siteProperties.getProperty('typesLinkToFolderContentsInFC'))
    if 'FormFolder' in typesLinkToFolderContentsInFC:
        typesLinkToFolderContentsInFC.remove('FormFolder')
    if 'FieldsetFolder' in typesLinkToFolderContentsInFC:
        typesLinkToFolderContentsInFC.remove('FieldsetFolder')
    siteProperties.manage_changeProperties(typesLinkToFolderContentsInFC = typesLinkToFolderContentsInFC)
    print >> out, "Removed FormFolder and FieldsetFolder from typesLinkToFolderContentsInFC"

    # Remove skin directory from skin selections
    removeSkinLayer(self, 'PloneFormGen')
    removeSkinLayer(self, 'PloneFormGenPlone3')
    # skinstool = getToolByName(self, 'portal_skins')
    # for skinName in skinstool.getSkinSelections():
    #     path = skinstool.getSkinPath(skinName)
    #     path = [i.strip() for i in  path.split(',')]
    #     for alayer in ('PloneFormGen', 'PloneFormGenPlone3',):
    #         if alayer in path:
    #             path.remove(alayer)
    #     path = ','.join(path)
    #     skinstool.addSkinSelection(skinName, path)
    print >> out, "Removed PloneFormGen layers from all skin selections"

    if HAS_PLONE30:
        kssRegTool = getToolByName(self, 'portal_kss', None)
        kssRegTool.manage_removeKineticStylesheet('ploneformgen.kss')
        print >> out, "Removed ploneformgen.kss from kss registry"

    print >> out, """\nNOTE: portal_properties/ploneformgen_properties
    was left in place so that you may reinstall without losing site
    configuration data. It's harmless, but feel free to delete it if
    you're sure you don't need any pfg site configuration changes."""

    print >> out, "\nSuccessfully uninstalled %s." % PROJECTNAME
    return out.getvalue()



""" A form action adapter that saves form submissions for download """

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo

from BTrees.IOBTree import IOBTree

from Products.CMFCore.permissions import View, ModifyPortalContent
from Products.CMFPlone.utils import base_hasattr, safe_hasattr

from Products.Archetypes.public import *
from Products.Archetypes.utils import contentDispositionHeader
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.base import registerATCT

from Products.PloneFormGen import HAS_PLONE30
from Products.PloneFormGen.config import *
from Products.PloneFormGen.content.actionAdapter import \
    FormActionAdapter, FormAdapterSchema

import logging

from DateTime import DateTime
import csv
from StringIO import StringIO
from types import StringTypes

try:
    # 3.0+
    from zope.contenttype import guess_content_type
except ImportError:
    try:
        # 2.5
        from zope.app.content_types import guess_content_type
    except ImportError:
        # 2.1
        from OFS.content_types import guess_content_type

logger = logging.getLogger("PloneFormGen")    


ExLinesField = LinesField


class FormSaveDataAdapter(FormActionAdapter):
    """A form action adapter that will save form input data and 
       return it in csv- or tab-delimited format."""

    schema = FormAdapterSchema.copy() + Schema((
        LinesField('ExtraData',
            widget=MultiSelectionWidget(
                label='Extra Data',
                description="""
                    Pick any extra data you'd like saved with the form input.
                    """,
                format='checkbox',
                i18n_domain = "ploneformgen",
                label_msgid = "label_savedataextra_text",
                description_msgid = "help_savedataextra_text",
                ),
            vocabulary = 'vocabExtraDataDL',
            ),
        StringField('DownloadFormat',
            searchable=0,
            required=1,
            default='csv',
            vocabulary = 'vocabFormatDL',
            widget=SelectionWidget(
                label='Download Format',
                i18n_domain = "ploneformgen",
                label_msgid = "label_downloadformat_text",
                ),
            ),
        BooleanField("UseColumnNames",
            required=False,
            searchable=False,
            widget=BooleanWidget(
                label = "Include Column Names",
                description = "Do you wish to have column names on the first line of downloaded input?",
                i18n_domain = "ploneformgen",
                label_msgid = "label_usecolumnnames_text",
                description_msgid = "help_usecolumnnames_text",
                ),
            ),
        ExLinesField('SavedFormInput',
            edit_accessor='getSavedFormInputForEdit',
            mutator='setSavedFormInput',
            searchable=0,
            required=0,
            primary=1,
            schemata="saved data",
            read_permission=DOWNLOAD_SAVED_PERMISSION,
            widget=TextAreaWidget(
                label="Saved Form Input",
                i18n_domain = "ploneformgen",
                label_msgid = "label_savedatainput_text",
                description_msgid = "help_savedatainput_text",
                ),
            ),
    ))

    schema.moveField('execCondition', pos='bottom')
    if not HAS_PLONE30:
        finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)

    meta_type      = 'FormSaveDataAdapter'
    portal_type    = 'FormSaveDataAdapter'
    archetype_name = 'Save Data Adapter'

    if HAS_PLONE30:
        immediate_view = 'fg_savedata_view_p3'
        default_view   = 'fg_savedata_view_p3'
        suppl_views    = ('fg_savedata_tabview_p3', 'fg_savedata_recview_p3',)
    else:
        immediate_view = 'fg_savedata_view'
        default_view   = 'fg_savedata_view'
        suppl_views    = ('fg_savedata_tabview', 'fg_savedata_recview',)

    security       = ClassSecurityInfo()


    def initializeArchetype(self, **kwargs):
        """ the trick of using a textarea widget with a lines field
            is causing a bad interpretation of the default. (It's
            getting represented as a string.)
        """

        FormActionAdapter.initializeArchetype(self, **kwargs)
        
        self.SavedFormInput = []


    def _migrateStorage(self):
        # we're going to use an IOBTree for storage. we need to
        # consider the possibility that self is from an
        # older version that uses the native Archetypes storage
        # in the SavedFormInput field.
        
        if not base_hasattr(self, '_inputStorage'):
            self._inputStorage = IOBTree()
            self._inputItems = 0
        
        if len(self.SavedFormInput):
            for row in self.SavedFormInput:
                self._inputStorage[self._inputItems] = row
                self._inputItems += 1
            self.SavedFormInput = []                


    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'getSavedFormInput')
    def getSavedFormInput(self):
        """ returns saved input as an iterable;
            each row is a sequence of fields.
        """

        if base_hasattr(self, '_inputStorage'):
            return self._inputStorage.values()
        else:
            return self.SavedFormInput


    security.declareProtected(ModifyPortalContent, 'getSavedFormInputForEdit')
    def getSavedFormInputForEdit(self, **kwargs):
        """ returns saved as CSV text """

        sbuf = StringIO()
        writer = csv.writer(sbuf)
        for row in self.getSavedFormInput():
            writer.writerow( row )
        res = sbuf.getvalue()
        sbuf.close()

        return res


    security.declareProtected(ModifyPortalContent, 'setSavedFormInput')
    def setSavedFormInput(self, value, **kwargs):
        """ expects value as csv text string, stores as list of lists """

        self._migrateStorage()

        self._inputStorage.clear()
        self._inputItems = 0

        if len(value):
            sbuf = StringIO( value )
            reader = csv.reader(sbuf)
            for row in reader:
                if row:
                    self._inputStorage[self._inputItems] = row
                    self._inputItems += 1
            sbuf.close()

        # logger.debug("setSavedFormInput: %s items" % self._inputItems)


    security.declareProtected(ModifyPortalContent, 'clearSavedFormInput')
    def clearSavedFormInput(self, **kwargs):
        """ convenience method to clear input buffer """

        self._migrateStorage()

        self._inputStorage.clear()
        self._inputItems = 0


    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'getSavedFormInputById')
    def getSavedFormInputById(self, id):
        """ Return the data stored for record with 'id' """
        lst =  [field.replace('\r','').replace('\n', r'\n') for field in self._inputStorage[id-1]]
        return lst

 
    security.declareProtected(ModifyPortalContent, 'manage_saveData')
    def manage_saveData(self, id,  data):
        """ Save the data for record with 'id' """

        self._migrateStorage()

        lst = list()
        for i in range(0, len(self.getColumnNames())):
            lst.append(getattr(data, 'item-%d' % i, '').replace(r'\n', '\n'))
 
        self._inputStorage[id-1] = lst
        self.REQUEST.RESPONSE.redirect(self.absolute_url() + '/view')


    security.declareProtected(ModifyPortalContent, 'manage_deleteData')
    def manage_deleteData(self, id):
        """ Delete the data for record with 'id' """

        self._migrateStorage()

        for i in range(id, self._inputItems):
            self._inputStorage[i-1] = self._inputStorage[i]
        del self._inputStorage[self._inputItems-1]
        self._inputItems -= 1
        
        self.REQUEST.RESPONSE.redirect(self.absolute_url() + '/view')


    def _addDataRow(self, value):

        self._migrateStorage()

        self._inputStorage[self._inputItems] = value
        self._inputItems += 1


    security.declareProtected(ModifyPortalContent, 'addDataRow')
    def addDataRow(self, value):
        """ a wrapper for the _addDataRow method """
        
        self._addDataRow(value)

    
    def onSuccess(self, fields, REQUEST=None, loopstop=False):
        """
        saves data.
        """

        if LP_SAVE_TO_CANONICAL and not loopstop:
            # LinguaPlone functionality:
            # check to see if we're in a translated
            # form folder, but not the canonical version.
            parent = self.aq_parent
            if safe_hasattr(parent, 'isTranslation') and \
               parent.isTranslation() and not parent.isCanonical():
                # look in the canonical version to see if there is
                # a matching (by id) save-data adapter.
                # If so, call its onSuccess method
                cf = parent.getCanonical()
                target = cf.get(self.getId())
                if target.meta_type == 'FormSaveDataAdapter':
                    target.onSuccess(fields, REQUEST, loopstop=True)
                    return

        from ZPublisher.HTTPRequest import FileUpload

        data = []
        for f in fields:
            if f.isFileField():
                file = REQUEST.form.get('%s_file' % f.fgField.getName())
                if isinstance(file, FileUpload) and file.filename != '':
                    file.seek(0)
                    fdata = file.read()
                    filename = file.filename
                    mimetype, enc = guess_content_type(filename, fdata, None)
                    if mimetype.find('text/') >= 0:
                        # convert to native eols
                        fdata = fdata.replace('\x0d\x0a', '\n').replace('\x0a', '\n').replace('\x0d', '\n')
                        data.append( '%s:%s:%s:%s' %  (filename, mimetype, enc, fdata) )
                    else:
                        data.append( '%s:%s:%s:Binary upload discarded' %  (filename, mimetype, enc) )
                else:
                    data.append( 'NO UPLOAD' )
            elif not f.isLabel():
                val = REQUEST.form.get(f.fgField.getName(),'')
                if not type(val) in StringTypes:
                    # Zope has marshalled the field into
                    # something other than a string
                    val = str(val)
                data.append(val)

        if self.ExtraData:
            for f in self.ExtraData:
                if f == 'dt':
                    data.append( str(DateTime()) )
                else:
                    data.append( getattr(REQUEST, f, '') )


        self._addDataRow( data )


    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'getColumnNames')
    def getColumnNames(self):
        """Returns a list of column names"""
        
        names = [field.getName() for field in self.fgFields(displayOnly=True)]
        for f in self.ExtraData:
            names.append(f)
        
        return names
        

    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'getColumnTitles')
    def getColumnTitles(self):
        """Returns a list of column titles"""
        
        names = [field.widget.label for field in self.fgFields(displayOnly=True)]
        for f in self.ExtraData:
            names.append(self.vocabExtraDataDL().getValue(f, ''))
        
        return names
        

    def _cleanInputForTSV(self, value):
        # make data safe to store in tab-delimited format

        return  str(value).replace('\x0d\x0a', r'\n').replace('\x0a', r'\n').replace('\x0d', r'\n').replace('\t', r'\t')


    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'download_tsv')
    def download_tsv(self, REQUEST=None, RESPONSE=None):
        """Download the saved data
        """

        filename = self.id
        if filename.find('.') < 0:
            filename = '%s.tsv' % filename
        header_value = contentDispositionHeader('attachment', self.getCharset(), filename=filename)
        RESPONSE.setHeader("Content-Disposition", header_value)
        RESPONSE.setHeader("Content-Type", 'text/tab-separated-values;charset=%s' % self.getCharset())

        if getattr(self, 'UseColumnNames', False):
            res = "%s\n" % '\t'.join( self.getColumnNames() )
            if isinstance(res, unicode):
                res = res.encode(self.getCharset())
        else:
            res = ''

        for row in self.getSavedFormInput():
            res = '%s%s\n' % (res, '\t'.join( [self._cleanInputForTSV(col) for col in row] ))

        return res


    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'download_csv')
    def download_csv(self, REQUEST=None, RESPONSE=None):
        """Download the saved data
        """

        filename = self.id
        if filename.find('.') < 0:
            filename = '%s.csv' % filename
        header_value = contentDispositionHeader('attachment', self.getCharset(), filename=filename)
        RESPONSE.setHeader("Content-Disposition", header_value)
        RESPONSE.setHeader("Content-Type", 'text/comma-separated-values;charset=%s' % self.getCharset())

        if getattr(self, 'UseColumnNames', False):
            res = "%s\n" % ','.join( self.getColumnNames() )
            if isinstance(res, unicode):
                res = res.encode(self.getCharset())
        else:
            res = ''

        return '%s%s' % (res, self.getSavedFormInputForEdit())


    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'download')
    def download(self, REQUEST=None, RESPONSE=None):
        """Download the saved data
        """

        format = getattr(self, 'DownloadFormat', 'tsv')
        if format == 'tsv':
            return self.download_tsv(REQUEST, RESPONSE)
        else:
            assert format == 'csv', 'Unknown download format'
            return self.download_csv(REQUEST, RESPONSE)


    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'rowAsColDict')
    def rowAsColDict(self, row, cols):
        """ Where row is a data sequence and cols is a column name sequence,
            returns a dict of colname:column. This is a convenience method
            used in the record view.
        """
    
        colcount = len(cols)

        rdict = {}
        for i in range(0, len(row)):
            if i < colcount:
                rdict[cols[i]] = row[i]
            else:
                rdict['column-%s' % i] = row[i]
        return rdict


    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'inputAsDictionaries')
    def inputAsDictionaries(self):
        """returns saved data as an iterable of dictionaries
        """

        cols = self.getColumnNames()

        for row in self.getSavedFormInput():
            yield self.rowAsColDict(row, cols)
        

    # alias for old mis-naming
    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'InputAsDictionaries')
    InputAsDictionaries = inputAsDictionaries


    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'formatMIME')
    def formatMIME(self):
        """MIME format selected for download
        """

        format = getattr(self, 'DownloadFormat', 'tsv')
        if format == 'tsv':
            return 'text/tab-separated-values'
        else:
            assert format == 'csv', 'Unknown download format'
            return 'text/comma-separated-values'


    security.declareProtected(DOWNLOAD_SAVED_PERMISSION, 'itemsSaved')
    def itemsSaved(self):
        """Download the saved data
        """

        if base_hasattr(self, '_inputItems'):
            return self._inputItems
        else:
            return len(self.SavedFormInput)


    def vocabExtraDataDL(self):
        """ returns vocabulary for extra data """

        return DisplayList( (
                ('dt',
                    self.translate( msgid='vocabulary_postingdt_text',
                    domain='ploneformgen',
                    default='Posting Date/Time')
                    ),
                ('HTTP_X_FORWARDED_FOR','HTTP_X_FORWARDED_FOR',),
                ('REMOTE_ADDR','REMOTE_ADDR',),
                ('HTTP_USER_AGENT','HTTP_USER_AGENT',),
                ) )


    def vocabFormatDL(self):
        """ returns vocabulary for format """

        return DisplayList( (
                ('tsv',
                    self.translate( msgid='vocabulary_tsv_text',
                    domain='ploneformgen',
                    default='Tab-Separated Values')
                    ),
                ('csv',
                    self.translate( msgid='vocabulary_csv_text',
                    domain='ploneformgen',
                    default='Comma-Separated Values')
                    ),
            ) )



registerATCT(FormSaveDataAdapter, PROJECTNAME)



""" Form fields, implemented via Archetypes Fields, Validators and Widgets"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from Products.Archetypes.public import *
from Products.Archetypes.utils import shasattr

from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.configuration import zconf

from Products.TALESField import TALESString, TALESLines

from Products.PloneFormGen.config import PROJECTNAME, EDIT_TALES_PERMISSION

from Products.CMFCore.permissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

from AccessControl import ClassSecurityInfo

from Products.PloneFormGen import PloneFormGenMessageFactory as _
from Products.PloneFormGen import HAS_PLONE25
from Products.PloneFormGen.widgets import RichLabelWidget

from Products.PloneFormGen.content.fieldsBase import *
# \
#    BaseFieldSchema, BareFieldSchema, BaseFieldSchemaLinesDefault, BaseFieldSchemaTextDefault, \
#    BaseFormField, BaseFieldSchemaStringDefault, \
#    maxlengthField, sizeField, vocabularyField, vocabularyOverrideField

from types import StringTypes, BooleanType
from DateTime import DateTime
import cgi

class FGStringField(BaseFormField):
    """ A string entry field """

    security  = ClassSecurityInfo()

    schema = BaseFieldSchemaStringDefault.copy() + Schema((
        maxlengthField,
        sizeField,
        StringField('fgStringValidator',
            vocabulary='stringValidatorsDL',
            enforceVocabulary=1,
            widget=SelectionWidget(label='Validator',
                description="""Tests input against simple string patterns.""",
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgstringvalidator_text",
                description_msgid = "help_fgstringvalidator_text",
                ),
        ),
    ))

    # hide references & discussion
    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)

    # Standard content type setup
    portal_type = meta_type = 'FormStringField'
    archetype_name = 'String Field'
    content_icon = 'StringField.gif'
    typeDescription= 'A string field'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = StringField('fg_string_field',
            searchable=0,
            required=0,
            write_permission = View,
            validators=('isNotTooLong',),
            )


    def stringValidatorsDL(self):
        """ return a display list of string validators.
        """

        fgt = getToolByName(self, 'formgen_tool')
        return fgt.getStringValidatorsDL()


    def setFgStringValidator(self, value, **kw):
        """ set simple validator """

        fgt = getToolByName(self, 'formgen_tool')

        if value and (value != 'vocabulary_none_text'):
            fgtid = fgt.stringValidators[value].get('id')
            if fgtid:
                self.fgField.validators = ('isNotTooLong', fgtid)
        else:
            self.fgField.validators = ('isNotTooLong',)
        self.fgField._validationLayer()

        self.fgStringValidator = value


registerATCT(FGStringField, PROJECTNAME)


class FGPasswordField(FGStringField):
    """ Password entry field (input is masked) """

    schema = BaseFieldSchemaStringDefault.copy() + Schema((
        maxlengthField,
        sizeField,
    ))

    # 'hidden' isn't really useful for this field.
    del schema['hidden']

    # hide references & discussion
    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)

    # Standard content type setup
    portal_type = meta_type = 'FormPasswordField'
    archetype_name = 'Password Field'
    content_icon = 'PasswordField.gif'
    typeDescription= 'A password field (input masked)'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute.
        #
        # We use our own widget and reset several widget options
        # so that the password value won't be lost if form
        # validation fails.
        self.fgField = StringField('fg_string_field',
            searchable=0,
            required=0,
            write_permission = View,
            validators=('isNotTooLong',),
            widget=PasswordWidget(
                macro='pfg_password',
                populate = True,
                postback = True,
                blurrable = True,
            ),
        )

registerATCT(FGPasswordField, PROJECTNAME)


class FGIntegerField(BaseFormField):
    """ Integer entry field """

    security  = ClassSecurityInfo()

    schema = BaseFieldSchemaStringDefault.copy() + Schema((
        IntegerField('minval',
            searchable=0,
            required=1,
            default='0',
            widget=IntegerWidget(
                label="Minimum Acceptable Value",
                description="""
                    The form will not accept values less than the number you enter here.
                """,
                i18n_domain = "ploneformgen",
                label_msgid = "label_minval_text",
                description_msgid = "help_minval_text",
                ),
            ),
        IntegerField('maxval',
            searchable=0,
            required=1,
            default='10000',
            widget=IntegerWidget(
                label="Maximum Acceptable Value",
                description="""
                    The form will not accept values greater than the number you enter here.
                """,
                i18n_domain = "ploneformgen",
                label_msgid = "label_maxval_text",
                description_msgid = "help_maxval_text",
                ),
            ),
        maxlengthField,
        sizeField,
    ))

    # 'hidden' isn't really useful for this field.
    del schema['hidden']

    # hide references & discussion
    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)

    # Standard content type setup
    portal_type = meta_type = 'FormIntegerField'
    archetype_name = 'Integer Field'
    content_icon = 'IntegerField.gif'
    typeDescription= 'A integer field'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = IntegerField('fg_integer_field',
            searchable=0,
            required=0,
            minval=0,
            maxval=10000,
            validators=('isNotTooLong', 'inExNumericRange',),
            write_permission = View,
            )

registerATCT(FGIntegerField, PROJECTNAME)


class FGFixedPointField(BaseFormField):
    """ Fixed-Point (float) entry field """

    security  = ClassSecurityInfo()

    schema = BaseFieldSchemaStringDefault.copy() + Schema((
        FixedPointField('minval',
            searchable=0,
            required=0,
            default='0.0',
            widget=DecimalWidget(
                label="Minimum Acceptable Value",
                description="""
                    The form will not accept values less than the number you enter here.
                """,
                size=8,
                i18n_domain = "ploneformgen",
                label_msgid = "label_minval_text",
                description_msgid = "help_minval_text",
                ),
            ),
        FixedPointField('maxval',
            searchable=0,
            required=1,
            default='10000.0',
            widget=DecimalWidget(
                label="Maximum Acceptable Value",
                description="""
                    The form will not accept values greater than the number you enter here.
                """,
                size=8,
                i18n_domain = "ploneformgen",
                label_msgid = "label_maxval_text",
                description_msgid = "help_maxval_text",
                ),
            ),
        maxlengthField,
        sizeField,
    ))

    # 'hidden' isn't really useful for this field.
    del schema['hidden']

    # and, required has only limited use ...
    schema['required'].widget.description = \
        """NOTE: For a fixed-point field, the required flag will not allow
           entry of a '0' value.
        """
    schema['required'].widget.description_msgid = "help_fprequired_text"

    # hide references & discussion
    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)

    # Standard content type setup
    portal_type = meta_type = 'FormFixedPointField'
    archetype_name = 'Fixed-Point Field'
    content_icon = 'FloatField.gif'
    typeDescription= 'A fixed-point (float) field'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = FixedPointField('fg_date_field',
            searchable=0,
            required=0,
            write_permission = View,
            validators=('isNotTooLong', 'inExNumericRange',),
            )

#    security.declareProtected(ModifyPortalContent, 'setThousands_commas')
#    def setThousands_commas(self, value, **kw):
#        """ set widget's thousands_commas """
#
#        self.fgField.widget.thousands_commas = value == '1'
#
#
#    security.declareProtected(ModifyPortalContent, 'getThousands_commas')
#    def getThousands_commas(self, **kw):
#        """ get widget's thousands_commas """
#
#        return self.fgField.widget.thousands_commas

registerATCT(FGFixedPointField, PROJECTNAME)


class FGBooleanField(BaseFormField):
    """ Boolean (checkbox) field """

    security  = ClassSecurityInfo()

    schema = BaseFieldSchema.copy() + Schema((
        StringField('fgDefault',
            searchable=0,
            required=0,
            widget=BooleanWidget(
                label='Default',
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgdefault_text",
                description_msgid = "help_fgdefault_text",
                ),
        ),
        StringField('fgBooleanValidator',
            vocabulary='boolVocabDL',
            enforceVocabulary=1,
            widget=SelectionWidget(label='Validator',
                description="""Choose a validator to require a particular response.""",
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgbooleanvalidator_text",
                description_msgid = "help_fgbooleanvalidator_text",
                ),
        ),
        StringField('fgBoolTrueString',
            required=0,
            searchable=0,
            default='1',
            widget=StringWidget(
                label="True Display String",
                description="""String to use in thanks page and mail when the field's checkbox is checked.""",
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgbooleantruestring_text",
                description_msgid = "help_fgbooleantruestring_text",
                ),
            ),
        StringField('fgBoolFalseString',
            required=0,
            searchable=0,
            default='0',
            widget=StringWidget(
                label="False Display String",
                description="""String to use in thanks page and mail when the field's checkbox is unchecked.""",
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgbooleanfalsestring_text",
                description_msgid = "help_fgbooleanfalsestring_text",
                ),
            ),
    ))
    schema['required'].widget.description = \
        """NOTE: For a checkbox field, the required flag doesn't do anything beyond
           putting a 'required' marker next to the label. If you wish to require a
           particular input, choose a validator below.
        """
    schema['required'].widget.description_msgid = "help_boolrequired_text"

    # 'hidden' isn't really useful for this field.
    del schema['hidden']

    # hide references & discussion
    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)

    # Standard content type setup
    portal_type = meta_type = 'FormBooleanField'
    archetype_name = 'Boolean Field'
    content_icon = 'CheckBoxField.gif'
    typeDescription= 'A CheckBox (Boolean) field'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = BooleanField('fg_boolean_field',
            searchable=0,
            required=0,
            write_permission = View,
            )

    security.declareProtected(ModifyPortalContent, 'setFgBooleanValidator')
    def setFgBooleanValidator(self, value, **kw):
        """ set boolean validator """

        if value:
            self.fgField.validators = (value,)
        else:
            self.fgField.validators = ()
        self.fgField._validationLayer()

        self.fgBooleanValidator = value

    def boolVocabDL(self):
        """ returns DisplayList of vocabulary for fgBooleanValidator """

        # Check for Plone 2.5
        if HAS_PLONE25:
          return DisplayList( (
                ('',
                    _(u'vocabulary_none_text', u'None')
                    ),
                ('isChecked',
                    _(u'vocabulary_ischecked_text', u'Is checked')
                    ),
                ('isUnchecked',
                    _(u'vocabulary_isnotchecked_text', u'Is not checked')
                    ),
                ) )
        else:
            return DisplayList( (
                ('',
                    self.translate( msgid='vocabulary_none_text',
                    domain='ploneformgen',
                    default='None')
                    ),
                ('isChecked',
                    self.translate( msgid='vocabulary_ischecked_text',
                    domain='ploneformgen',
                    default='Is checked')
                    ),
                ('isUnchecked',
                    self.translate( msgid='vocabulary_isnotchecked_text',
                    domain='ploneformgen',
                    default='Is not checked')
                    ),
                ) )


    def htmlValue(self, REQUEST):
        """ Return value instead of key """

        value = REQUEST.form.get(self.__name__, 'No Input')
        if type(value) == BooleanType:
            if value:
                return self.fgBoolTrueString
        elif value == '1':
            return self.fgBoolTrueString

        return self.fgBoolFalseString



registerATCT(FGBooleanField, PROJECTNAME)


class FGDateField(BaseFormField):
    """ Date/Time Entry Field """

    security  = ClassSecurityInfo()

    schema = BaseFieldSchemaStringDefault.copy() + Schema((
        BooleanField('fgShowHM',
            searchable=0,
            required=0,
            default=1,
            widget=BooleanWidget(
                label='Show Time Selection Options',
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgshowhm_text",
                description_msgid = "help_fgshowhm_text",
                ),
        ),
        IntegerField('fgStartingYear',
            searchable=0,
            required=0,
            default='1999',
            widget=IntegerWidget(
                label='Starting Year',
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgstartingyear_text",
                description = "The first year to offer in the year drop-down.",
                description_msgid = "help_fgstartingyear_text",
                ),
        ),
        IntegerField('fgEndingYear',
            searchable=0,
            required=0,
            default=None,
            widget=IntegerWidget(
                label='Ending Year',
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgendingyear_text",
                description = """The last year to offer in the year drop-down.
                 Leave this empty if you wish to instead use a number of future years.""",
                description_msgid = "help_fgendingyear_text",
                ),
        ),
        IntegerField('fgFutureYears',
            searchable=0,
            required=0,
            default='5',
            widget=IntegerWidget(
                label='Future Years To Display',
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgfutureyears_text",
                description = """The number of future years to offer in the year drop-down.
                 (This is only applicable if you have not specified an ending year.)""",
                description_msgid = "help_fgfutureyears_text",
                ),
        ),
))

    # 'hidden' isn't really useful for this field.
    del schema['hidden']

    # hide references & discussion
    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)

    # Standard content type setup
    portal_type = meta_type = 'FormDateField'
    archetype_name = 'Date/Time Field'
    content_icon = 'DateTimeField.gif'
    typeDescription= 'A date/time field. Time component is optional.'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = DateTimeField('fg_date_field',
            searchable=0,
            required=0,
            write_permission = View,
            widget=CalendarWidget(),
            )


    security.declareProtected(ModifyPortalContent, 'setFgShowHM')
    def setFgShowHM(self, value, **kw):
        """ set show_hm """

        if type(value) == BooleanType:
            self.fgField.widget.show_hm = value
        else:
            self.fgField.widget.show_hm = value == '1'

        self.fgShowHM = value


    security.declareProtected(ModifyPortalContent, 'setFgStartingYear')
    def setFgStartingYear(self, value, **kw):
        """ set starting_year """

        if value:
            try:
                self.fgField.widget.starting_year = int(value)
                self.fgStartingYear = value
            except ValueError:
                pass
        else:
            self.fgField.widget.starting_year = None
            self.fgStartingYear = value            
            

    security.declareProtected(ModifyPortalContent, 'setFgEndingYear')
    def setFgEndingYear(self, value, **kw):
        """ set ending_year """

        if value:
            try:
                self.fgField.widget.ending_year = int(value)
                self.fgEndingYear = value
            except ValueError:
                pass
        else:
                self.fgField.widget.ending_year = None
                self.fgEndingYear = value


    security.declareProtected(ModifyPortalContent, 'setFgFutureYears')
    def setFgFutureYears(self, value, **kw):
        """ set future_years """

        if value:
            try:
                self.fgField.widget.future_years = int(value)
                self.fgFutureYears = value
            except ValueError:
                pass
        else:
            self.fgField.widget.future_years = None
            self.fgFutureYears = value            


    def _toLocalizedTime(self, time, long_format=None):
        tool = getToolByName(self, 'translation_service')
        return tool.ulocalized_time(time, long_format=long_format)


    def htmlValue(self, REQUEST):
        """ return from REQUEST, this field's value, rendered as XHTML.
        """

        value = REQUEST.form.get(self.__name__, 'No Input')

        # The replace('-','/') keeps the DateTime routine from
        # interpreting this as UTC. Odd, but true.

        try:
            dt = DateTime(value.replace('-','/'))
        except (DateTime.SyntaxError, DateTime.DateError):
            # probably better to simply return the input
            return cgi.escape(value)

        if self.fgField.widget.show_hm:
            ppt = getToolByName(self, 'portal_properties')
            long_format = ppt.site_properties.localLongTimeFormat
            value = self._toLocalizedTime(dt, long_format=long_format)
        else:
            value = self._toLocalizedTime(dt)

        return cgi.escape(value)


    def specialValidator(self, value, field, REQUEST, errors):
        """ Archetypes isn't validating non-required dates --
            so we need to.
        """

        fname = field.getName()
        month = REQUEST.form.get('%s_month'%fname, '01')
        day = REQUEST.form.get('%s_month'%fname, '01')
        
        if (month == '00') and (day == '00'):
            value = ''
            REQUEST.form[fname] = ''
        
        if value and not field.required:
            try:
                dt = DateTime(value)
            except (DateTime.SyntaxError, DateTime.DateError):
                return "Validation failed(isValidDate): this is not a valid date."
        return 0        


registerATCT(FGDateField, PROJECTNAME)


class FGLabelField(BaseFormField):
    """ Label-Only field (no input component) """

    security  = ClassSecurityInfo()

    schema = BareFieldSchema.copy()
    # hide references & discussion
    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)

    # Standard content type setup
    portal_type = meta_type = 'FormLabelField'
    archetype_name = 'Label Field'
    content_icon = 'LabelField.gif'
    typeDescription= 'A Label (No Input) field'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = StringField('fg_label_field',
            searchable=0,
            required=0,
            write_permission = View,
            widget=LabelWidget(),
            )

    def isLabel(self):
        return True

registerATCT(FGLabelField, PROJECTNAME)


class FGLinesField(BaseFormField):
    """ A line entry field. This appears as a textarea without wordwrap. """

    security  = ClassSecurityInfo()

    schema = BaseFieldSchemaLinesDefault.copy()

    # Standard content type setup
    portal_type = meta_type = 'FormLinesField'
    archetype_name = 'Lines Field'
    content_icon = 'LinesField.gif'
    typeDescription= 'A lines field. This appears as a textarea without wordwrap.'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = LinesField('fg_lines_field',
            searchable=0,
            required=0,
            write_permission = View,
            )

    def fgPrimeDefaults(self, request, contextObject=None):
        """ primes request with default """

        # the lines widget will choke if there is no value
        BaseFormField.fgPrimeDefaults(self, request, contextObject)
        request.form.setdefault(self.fgField.__name__, ['',])


registerATCT(FGLinesField, PROJECTNAME)


class FGSelectionField(BaseFormField):
    """ Selection Field (radio buttons or select) """

    security  = ClassSecurityInfo()

    schema = BaseFieldSchemaStringDefault.copy() + Schema((
        vocabularyField,
        vocabularyOverrideField,
        StringField('fgFormat',
            searchable=0,
            required=0,
            default='flex',
            enforceVocabulary=1,
            vocabulary='formatVocabDL',
            widget=SelectionWidget(
                label='Presentation Widget',
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgformat_text",
                description_msgid = "help_fgformat_text",
                ),
        ),
    ))

    # 'hidden' isn't really useful for a selection field.
    # Just use a hidden string field if you really need this.
    del schema['hidden']

    # hide references & discussion
    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)

    # Standard content type setup
    portal_type = meta_type = 'FormSelectionField'
    archetype_name = 'Selection Field'
    content_icon = 'ListField.gif'
    typeDescription= 'A selection field'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = StringVocabularyField('fg_selection_field',
            searchable=0,
            required=0,
            widget=SelectionWidget(),
            vocabulary = '_get_selection_vocab',
            enforceVocabulary=1,
            write_permission = View,
            )


    security.declareProtected(ModifyPortalContent, 'setFgFormat')
    def setFgFormat(self, value, **kw):
        """ set selection format """

        self.fgField.widget.format = value
        self.fgFormat = value


    def formatVocabDL(self):
        """ returns vocabulary for fgFormat """

        # Check for Plone 2.5
        if HAS_PLONE25:
          return DisplayList( (
                ('flex',
                        _(u'vocabulary_flex_text', u'Flexible (radio for short, select for longer)')
                    ),
                ('select',
                        _(u'vocabulary_selection_text', u'Selection list')
                    ),
                ('radio',
                        _(u'vocabulary_radio_text', u'Radio buttons')
                    ),
            ) )
        else:
            return DisplayList( (
                ('flex',
                    self.translate( msgid='vocabulary_flex_text',
                    domain='ploneformgen',
                    default='Flexible (radio for short, select for longer)')
                    ),
                ('select',
                    self.translate( msgid='vocabulary_selection_text',
                    domain='ploneformgen',
                    default='Selection list')
                    ),
                ('radio',
                    self.translate( msgid='vocabulary_radio_text',
                    domain='ploneformgen',
                    default='Radio buttons')
                    ),
            ) )


    def htmlValue(self, REQUEST):
        """ Return value instead of key """

        utils = getToolByName(self, 'plone_utils')
        charset = utils.getSiteEncoding()

        value = REQUEST.form.get(self.__name__, '')

        # note that vocabulary items are in unicode;
        # so, we must decode before lookup
        vu = value.decode(charset)

        vocabulary = self.fgField.Vocabulary(self)
        v = vocabulary.getValue(vu) or vu

        return cgi.escape(v.encode(charset))


registerATCT(FGSelectionField, PROJECTNAME)


class FGMultiSelectField(BaseFormField):
    """ Multiple selection field (select with multiple or check boxes) """

    security  = ClassSecurityInfo()

    schema = BaseFieldSchemaLinesDefault.copy() + Schema((
        vocabularyField,
        vocabularyOverrideField,
        StringField('fgFormat',
            searchable=0,
            required=0,
            default='select',
            enforceVocabulary=1,
            vocabulary='formatVocabDL',
            widget=SelectionWidget(
                label='Presentation Widget',
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgmsformat_text",
                description_msgid = "help_fgmsformat_text",
                ),
        ),
    ))

    # current Archetypes doesn't really support hidden for
    # multi-select. Use a lines field if you really need this
    del schema['hidden']

    # hide references & discussion
    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)

    # Standard content type setup
    portal_type = meta_type = 'FormMultiSelectionField'
    archetype_name = 'Multi-Select Field'
    content_icon = 'MultipleListField.gif'
    typeDescription= 'A multiple-selection field'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = LinesVocabularyField('fg_mselection_field',
            searchable=0,
            required=0,
            widget=MultiSelectionWidget(),
            vocabulary = '_get_selection_vocab',
            enforceVocabulary=1,
            write_permission = View,
            )


    security.declareProtected(ModifyPortalContent, 'setFgFormat')
    def setFgFormat(self, value, **kw):
        """ set selection format """

        self.fgField.widget.format = value
        self.fgFormat = value


    security.declareProtected(ModifyPortalContent, 'setFgRows')
    def setFgRows(self, value, **kw):
        """ sets widget rows """

        self.fgField.widget.size = value


    security.declareProtected(View, 'getFgRows')
    def getFgRows(self, **kw):
        """ gets widget rows """

        return self.fgField.widget.size


    def formatVocabDL(self):
        """ returns vocabulary for fgFormat """

        # Check for Plone 2.5
        if HAS_PLONE25:
          return DisplayList( (
            ('select',
                    _(u'vocabulary_selection_text', u'Selection list')
                ),
            ('checkbox',
                    _(u'vocabulary_checkbox_text', u'Checkbox list')
                )
            ) )
        else:
            return DisplayList( (
                ('select',
                    self.translate( msgid='vocabulary_selection_text',
                    domain='ploneformgen',
                    default='Selection list')
                    ),
                ('checkbox',
                    self.translate( msgid='vocabulary_checkbox_text',
                    domain='ploneformgen',
                    default='Checkbox list')
                    )
                ) )


    def htmlValue(self, REQUEST):
        """ Return value instead of key """

        utils = getToolByName(self, 'plone_utils')
        charset = utils.getSiteEncoding()

        value = REQUEST.form.get(self.__name__, [])

        vocabulary = self.fgField.Vocabulary(self)
        result = []
        for k in value:
            # there'll be an empty string to avoid
            if len(k):
                # vocabulary items are in unicode;
                # so decode the key before lookup
                ku = k.decode(charset)
                v = vocabulary.getValue(ku) or ku
                result.append(v)

        value = u', '.join(result)

        return cgi.escape(value.encode(charset))


registerATCT(FGMultiSelectField, PROJECTNAME)


class PlainTextField(TextField):
    """
    A text field forced text/plain.
    Without this hack, the textarea widget will (by acquisition) call
    the form folder's getContentType method.
    
    Also, the getAllowedContentTypes method override solves a problem
    in Plone 3 where old instances of a text field with None set for
    allowed_content_types were badly handled by the new getAllowedContentTypes
    method in TextField -- which used site mimetype defaults.
    """

    security  = ClassSecurityInfo()

    security.declarePublic('getContentType')
    def getContentType(self, instance, fromBaseUnit=True):
        return 'text/plain'

    security.declarePublic('getAllowedContentTypes')
    def getAllowedContentTypes(self, instance):
        return 'text/plain'        


class FGTextField(BaseFormField):
    """ Text (textarea) field """

    security  = ClassSecurityInfo()

    schema = BaseFieldSchemaTextDefault.copy()

    # Standard content type setup
    portal_type = meta_type = 'FormTextField'
    archetype_name = 'Text Field'
    content_icon = 'TextAreaField.gif'
    typeDescription= 'A text area field'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = PlainTextField('fg_text_field',
            searchable=0,
            required=0,
            write_permission = View,
            validators=('isNotTooLong',),
            default_content_type = 'text/plain',
            allowable_content_types = ('text/plain',),
            widget = TextAreaWidget(
                maxlength=0,
                ),
            )

    security.declareProtected(View, 'isBinary')
    def isBinary(self, key):
        return False;

    security.declareProtected(View, 'getContentType')
    def getContentType(self, key=None):
        return 'text/plain'
        

registerATCT(FGTextField, PROJECTNAME)


class HtmlTextField(TextField):
    """
    A text field forced text/html.
    Without this hack, the rich widget will (by acquisition) call
    the form folder's getContentType method.
    """

    security  = ClassSecurityInfo()

    security.declarePublic('getContentType')
    def getContentType(self, instance, fromBaseUnit=True):
        return 'text/html'


class FGRichTextField(BaseFormField):
    """ Rich-text (visual editor) field """

    from Products.ATContentTypes.config import HAS_MX_TIDY

    security  = ClassSecurityInfo()

    schema = BaseFieldSchemaRichTextDefault.copy()

    # 'hidden' isn't really useful for an RT field.
    # Just use a hidden string field if you really need this.
    del schema['hidden']

    if HAS_MX_TIDY:
        schema = schema + Schema((
            StringField('fgStringValidator',
                vocabulary='htmlValidatorsDL',
                enforceVocabulary=1,
                default='isTidyHtmlWithCleanup',
                widget=SelectionWidget(label='Validator',
                    description="""Input tests using HTMLTidy (if installed).""",
                    i18n_domain = "ploneformgen",
                    label_msgid = "label_fgstringvalidator_text",
                    description_msgid = "help_fgrtvalidator_text",
                    ),
                ),
        ))

    # Standard content type setup
    portal_type = meta_type = 'FormRichTextField'
    archetype_name = 'RichText Field'
    content_icon = 'RichTextField.gif'
    typeDescription= 'A rich text area field'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = HtmlTextField('fg_text_field',
            searchable=0,
            required=0,
            write_permission = View,
            validators = ('isNotTooLong', 'isTidyHtmlWithCleanup',),
            default_content_type = 'text/html',
            default_output_type = 'text/x-html-safe',
            allowable_content_types = zconf.ATDocument.allowed_content_types,
            widget = RichWidget(
                    allow_file_upload = False,
                    ),
            )


    security.declareProtected(View, 'isBinary')
    def isBinary(self, key):
        return False;

    security.declareProtected(View, 'getContentType')
    def getContentType(self, key=None):
        return 'text/html'

    def htmlValidatorsDL(self):
        """ return a display list of string validators.
            this is a hack for 118n
        """

        # Check for Plone 2.5
        if HAS_PLONE25:
            return DisplayList( (
                ('',
                    _(u'vocabulary_none_text', u'None')
                    ),
                ('isTidyHtml',
                    _(u'vocabulary_istidyhtml_text', u'Is Tidy HTML (fails on errors and warnings)')
                    ),
                ('isTidyHtmlWithCleanup',
                    _(u'vocabulary_istidyhtmlwithcleanup_text', u'Tidy HTML With Cleanup (fails on errors, cleans up rest)')
                    ),
                ) )
        else:
            return DisplayList( (
                ('',
                    self.translate( msgid='vocabulary_none_text',
                    domain='ploneformgen',
                    default='None')
                    ),
                ('isTidyHtml',
                    self.translate( msgid='vocabulary_istidyhtml_text',
                    domain='ploneformgen',
                    default='Is Tidy HTML (fails on errors and warnings)')
                    ),
                ('isTidyHtmlWithCleanup',
                    self.translate( msgid='vocabulary_istidyhtmlwithcleanup_text',
                    domain='ploneformgen',
                    default='Tidy HTML With Cleanup (fails on errors, cleans up rest)')
                    ),
                ) )

    def htmlValue(self, REQUEST):
        """ override, as this field is already html.
        """

        return REQUEST.form.get(self.__name__, 'No Input')


registerATCT(FGRichTextField, PROJECTNAME)


class FGRichLabelField(BaseFormField):
    """ Rich-text label field """

    security  = ClassSecurityInfo()

    schema = BareFieldSchema.copy() + Schema((
        TextField('fgDefault',
            searchable=0,
            required=0,
            validators = ('isTidyHtmlWithCleanup',),
            default_content_type = 'text/html',
            default_output_type = 'text/x-html-safe',
            allowable_content_types = zconf.ATDocument.allowed_content_types,
            widget=RichWidget(label='Label body',
                description="""
                    The text to display in the form.
                """,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fglabelbody_text",
                description_msgid = "help_fglabelbody_text",
                allow_file_upload = False,
                ),
            ),
        TALESString('fgTDefault',
            schemata='overrides',
            searchable=0,
            required=0,
            validators=('talesvalidator',),
            default='',
            write_permission=EDIT_TALES_PERMISSION,
            widget=StringWidget(label="Default Expression",
                description="""
                    A TALES expression that will be evaluated when the form is displayed
                    to get the field default value.
                    Leave empty if unneeded. Your expression should evaluate as a string.
                    PLEASE NOTE: errors in the evaluation of this expression will cause
                    an error on form display.
                """,
                size=70,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgtdefault_text",
                description_msgid = "help_fgtdefault_text",
                ),
            ),
        ))

    schema['title'].widget.label = "Title"
    schema['title'].widget.label_msgid='label_title'
    schema['title'].widget.description = "Not displayed on form."
    schema['title'].widget.description_msgid = 'help_notdisplayed_text'
    schema['description'].schemata = 'metadata'

    # Standard content type setup
    portal_type = meta_type = 'FormRichLabelField'
    archetype_name = 'Rich Label Field'
    content_icon = 'RichLabelField.gif'
    typeDescription= 'A rich-text label'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = HtmlTextField('fg_text_field',
            searchable=0,
            required=0,
            write_permission = View,
            default_content_type = 'text/html',
            default_output_type = 'text/x-html-safe',
            allowable_content_types = zconf.ATDocument.allowed_content_types,
            widget = RichLabelWidget(),
            )

    security.declareProtected(ModifyPortalContent, 'setFgDefault')
    def setFgDefault(self, value, **kw):
        """ set default for field """

        self.fgField.default = value

    def getRawFgDefault(self, **kw):
        """ get default for field """

        return self.fgField.default

    security.declareProtected(View, 'isBinary')
    def isBinary(self, key):
        return False;

    security.declareProtected(View, 'getContentType')
    def getContentType(self, key=None):
        return 'text/html'

    def isLabel(self):
        return True

registerATCT(FGRichLabelField, PROJECTNAME)


class FGFileField(BaseFormField):
    """ File (upload) field """

    security  = ClassSecurityInfo()

    schema = BaseFieldSchema.copy() + Schema((
        StringField('fgMaxMB',
            searchable=0,
            required=0,
            default=0,
            widget=IntegerWidget(
                label='Maximum Upload Size (Megabytes)',
                description="""Set to 0 for no limit.""",
                label_msgid = "label_filemaxmb_text",
                description_msgid = "help_filemaxmb_text",
                i18n_domain = "ploneformgen",
                ),
        ),

    ))

    # 'hidden' isn't really useful for a file field.
    del schema['hidden']

    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)

    # Standard content type setup
    portal_type = meta_type = 'FormFileField'
    archetype_name = 'File Field'
    content_icon = 'FileField.gif'
    typeDescription= 'A file field'

    def __init__(self, oid, **kwargs):
        """ initialize class """

        BaseFormField.__init__(self, oid, **kwargs)

        # set a preconfigured field as an instance attribute
        self.fgField = FileField('fg_file_field',
            searchable=0,
            required=0,
            write_permission = View,
            accessor = 'nullAccessor',
            maxsize=0,
            validators=('isMaxSize',),
            )

    def isFileField(self):
        return True

    security.declareProtected(ModifyPortalContent, 'setFgMaxMB')
    def setFgMaxMB(self, value, **kw):
        """ set the maxmb """

        self.fgField.maxsize = int(value)

    security.declareProtected(View, 'getFgMaxMB')
    def getFgMaxMB(self, **kw):
        """ get the maxmb """

        return self.fgField.maxsize

    def htmlValue(self, REQUEST):

        from ZPublisher.HTTPRequest import FileUpload
        from OFS.content_types import guess_content_type

        file = REQUEST.form.get('%s_file' % self.fgField.getName())
        if isinstance(file, FileUpload) and file.filename != '':
            file.seek(0)
            fdata = file.read()
            filename = file.filename
            mimetype, enc = guess_content_type(filename, fdata, None)
            return "%s: %s bytes" % (mimetype, len(fdata))
        else:
            return 'No Input'

    def getFieldFormName(self):
        """field widget appends '_file' file to field id"""

        return self.fgField.getName() + '_file'


registerATCT(FGFileField, PROJECTNAME)



#
# Zope 3.1-style messagefactory module for Zope <= 2.9 (Zope 3.1)
#

# BBB: Zope 2.8 / Zope X3.0
from zope.i18nmessageid import MessageIDFactory
msg_factory = MessageIDFactory('ploneformgen')

def PloneFormGenMessageFactory(ustr, default=None, mapping=None):
    message = msg_factory(ustr, default)
    if mapping is not None:
        message.mapping.update(mapping)
    return message




"""thanksPage -- A smart(er) thank-you page for PloneFormGen"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from zope.interface import implements

from Products.PloneFormGen.config import PROJECTNAME
from Products.PloneFormGen.interfaces import IPloneFormGenThanksPage

from AccessControl import ClassSecurityInfo

from Products.Archetypes.public import *
from Products.Archetypes.utils import shasattr
from Products.Archetypes.interfaces.field import IField

from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.configuration import zconf

from Products.CMFCore.permissions import View, ModifyPortalContent

from Products.PloneFormGen import PloneFormGenMessageFactory as _
from Products.PloneFormGen import HAS_PLONE25, HAS_PLONE30

if HAS_PLONE25:
  import zope.i18n


ThanksPageSchema = ATContentTypeSchema.copy() + Schema((
    BooleanField('showAll',
        required=0,
        searchable=0,
        default='1',
        widget=BooleanWidget(
            label="Show All Fields",
            description="""
                Check this to display input for all fields
                (except label and file fields). If you check
                this, the choices in the pick box below
                will be ignored.
                """,
            label_msgid = "label_showallfields_text",
            description_msgid = "help_showallfields_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    LinesField('showFields',
        required=0,
        searchable=0,
        vocabulary='fieldDisplayList',
        widget=PicklistWidget(
            label="Show Responses",
            description="""
                Pick the fields whose inputs you'd like to display on
                the success page.
                """,
            label_msgid = "label_showfields_text",
            description_msgid = "help_showfields_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    BooleanField('includeEmpties',
        required=0,
        searchable=0,
        default='1',
        widget=BooleanWidget(
            label="Include Empties",
            description="""
                Check this to display fields titles
                for fields that received no input. Uncheck
                to leave fields with no input off the list.
                """,
            label_msgid = "label_includeEmpties_text",
            description_msgid = "help_includeEmpties_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    TextField('thanksPrologue',
        schemata='decoration',
        required=False,
        searchable=False,
        primary=False,
        validators = ('isTidyHtmlWithCleanup',),
        default_content_type = zconf.ATDocument.default_content_type,
        default_output_type = 'text/x-html-safe',
        allowable_content_types = zconf.ATDocument.allowed_content_types,
        widget = RichWidget(
            label = "Thanks Prologue",
            label_msgid = "label_thanksprologue_text",
            description = "This text will be displayed above the selected field inputs.",
            description_msgid = "help_thanksprologue_text",
            rows = 8,
            i18n_domain = "ploneformgen",
            allow_file_upload = zconf.ATDocument.allow_document_upload,
            ),
        ),
    TextField('thanksEpilogue',
        schemata='decoration',
        required=False,
        searchable=False,
        primary=False,
        validators = ('isTidyHtmlWithCleanup',),
        default_content_type = zconf.ATDocument.default_content_type,
        default_output_type = 'text/x-html-safe',
        allowable_content_types = zconf.ATDocument.allowed_content_types,
        widget = RichWidget(
            label = "Thanks Epilogue",
            label_msgid = "label_thanksepilogue_text",
            description = "The text will be displayed after the field inputs.",
            description_msgid = "help_thanksepilogue_text",
            rows = 8,
            i18n_domain = "ploneformgen",
            allow_file_upload = zconf.ATDocument.allow_document_upload,
            ),
        ),
    TextField('noSubmitMessage',
        schemata='no-input',
        required=False,
        searchable=False,
        primary=False,
        validators = ('isTidyHtmlWithCleanup',),
        default_content_type = zconf.ATDocument.default_content_type,
        default_output_type = 'text/x-html-safe',
        default="""
            <p>No input was received. Please <a title="Test Folder" href=".">visit the form</a>.</p>
            """,
        allowable_content_types = zconf.ATDocument.allowed_content_types,
        widget = RichWidget(
            label = "No Submit Message",
            label_msgid = "label_nosubmit_text",
            description = """
                The text to display if the browser reaches this
                thanks page without submitting a form. Typically, this
                would direct the reader to the form.
                """,
            description_msgid = "help_nosubmit_text",
            rows = 4,
            i18n_domain = "ploneformgen",
            allow_file_upload = zconf.ATDocument.allow_document_upload,
            ),
        ),
    ))


finalizeATCTSchema(ThanksPageSchema, folderish=True, moveDiscussion=False)
if HAS_PLONE30:
    # As of P3.0, rich text fields on non-default schema
    # still don't function.
    # Reorganize schema as well as possible.
    ThanksPageSchema['thanksPrologue'].schemata = 'default'
    ThanksPageSchema['thanksEpilogue'].schemata = 'default'
    ThanksPageSchema['noSubmitMessage'].schemata = 'default'
    ThanksPageSchema['includeEmpties'].schemata = 'fields'
    ThanksPageSchema['showAll'].schemata = 'fields'
    ThanksPageSchema['showFields'].schemata = 'fields'
    # simplify schema
    for afield in ('subject', 
                   'relatedItems', 
                   'location', 
                   'language', 
                   'effectiveDate', 
                   'expirationDate', 
                   'creation_date', 
                   'modification_date', 
                   'creators', 
                   'contributors', 
                   'rights', 
                   'allowDiscussion', 
                   'excludeFromNav', ):
        ThanksPageSchema[afield].widget.visible = {'view':'invisible','edit':'invisible'}
        ThanksPageSchema[afield].schemata = 'default'

class FormThanksPage(ATCTContent):
    """A thank-you page that can display form input"""

    implements(IPloneFormGenThanksPage)

    schema         =  ThanksPageSchema

    content_icon   = 'ThanksPage.gif'
    meta_type      = 'FormThanksPage'
    portal_type    = 'FormThanksPage'
    archetype_name = 'Thanks Page'

    immediate_view = 'fg_thankspage_view'
    default_view   = 'fg_thankspage_view'
    suppl_views = ()

    typeDescription= 'A thank-you page that can display form input.'

    global_allow = 0    

    security       = ClassSecurityInfo()


    def initializeArchetype(self, **kwargs):
        """ Translate the adapter in the current langage
        """

        ATCTContent.initializeArchetype(self, **kwargs)

        if HAS_PLONE25:
            self.setNoSubmitMessage(zope.i18n.translate(_(u'pfg_thankspage_nosubmitmessage', u'<p>No input was received. Please <a title="Test Folder" href=".">visit the form</a>.</p>'), context=self.REQUEST))
        else:
            self.setNoSubmitMessage(self.translate(
                                  msgid='pfg_thankspage_nosubmitmessage',
                                  domain='ploneformgen',
                                  default='<p>No input was received. Please <a title="Test Folder" href=".">visit the form</a>.</p>'))


    security.declareProtected(View, 'fieldDisplayList')
    def fieldDisplayList(self):
        """ returns a DisplayList of all fields """
        
        return self.fgFieldsDisplayList()
        
    
    security.declareProtected(View, 'displayFields')
    def displayFields(self):
        """ Returns a list of fields that should be
            displayed on the thanks page.
        """

        if self.showAll:
            # acquire field list from parent
            return self.fgFields(displayOnly=True)
        
        # acquire field list from parent
        fields = self.fgFields()
        res = []
        for id in self.showFields:
            # inefficient if we get many fields
            for f in fields:
                if f.getName() == id:
                    res.append(f)
                    break
        return res
        

    security.declareProtected(View, 'displayInputs')
    def displayInputs(self, request):
        """ Returns sequence of dicts {'label':fieldlabel, 'value':input}
        """

        # get a list of all candidate fields
        myFields = []
        for obj in self.aq_parent._getFieldObjects():
            if not (IField.isImplementedBy(obj) or obj.isLabel()):
                myFields.append(obj)

        # Now, determine which fields we show
        if self.showAll:
            sFields = myFields
        else:
            sFields = []
            # acquire field list from parent
            res = []
            for id in self.showFields:
                # inefficient if we get many fields
                for f in myFields:
                    if f.getId() == id:
                        sFields.append(f)
                        break

        # Now, build the results list
        res = []
        for obj in sFields:
            value = obj.htmlValue(request)
            if self.includeEmpties or (value and (value != 'No Input')):
                res.append( {
                    'label' : obj.fgField.widget.label,
                    'value' : value, 
                    } )
            
        return res
        

    security.declareProtected(ModifyPortalContent, 'setShowFields')
    def setShowFields(self, value, **kw):
        """ Reorder form input to match field order """
        # This wouldn't be desirable if the PickWidget
        # retained order.
        
        self.showFields = []
        for field in self.fgFields():
            id = field.getName()
            if id in value:
                self.showFields.append(id)
        

registerATCT(FormThanksPage, PROJECTNAME)


from Products.validation import validation, interfaces

class IsCheckedValidator:
    """ Validates a Boolean field, which should have a "1" (checked)
        or "0" unchecked, to be checked.
    """

    __implements__ = (interfaces.ivalidator,)

    name = 'IsCheckedValidator'

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):

        # import pdb; pdb.set_trace()

        if value == '1':
            return 1

        return ("Validation failed(%s): must be checked." % self.name)

validation.register(IsCheckedValidator('isChecked'))


class IsUncheckedValidator:
    """ Validates a Boolean field, which should have a "1" (checked)
        or "0" unchecked, to be unchecked.
    """

    __implements__ = (interfaces.ivalidator,)

    name = 'IsUncheckedValidator'

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):

        # import pdb; pdb.set_trace()

        if value == '0':
            return 1

        return ("Validation failed(%s): must be unchecked." % self.name)

validation.register(IsUncheckedValidator('isUnchecked'))



"""validationMessages -- cleanup Archetypes/validation messages"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

# Messages taken from godchap's gotcha-validation-i18n branch of
# archetypes validation

from Products.CMFCore.utils import getToolByName

from Products.PloneFormGen import HAS_PLONE25


import re
from types import StringTypes

verrorRE = re.compile("Validation failed\((.+)\):")
enrMessageRE = re.compile( "Validation failed\(inExNumericRange\): could not convert '(.+)' to number" )
enrSmallMessageRE = re.compile( "Validation failed\(inExNumericRange\): '(.+?)' is too small. Must be at least (.+)." )
enrLargeMessageRE = re.compile( "Validation failed\(inExNumericRange\): '(.+?)' is too large. Must be no greater than (.+)." )
mlTooLongRE = re.compile( "Validation failed\(isNotTooLong\): '.+?' is too long. Must be no longer than (.+) characters." )

    
if HAS_PLONE25:

    from Products.PloneFormGen import PloneFormGenMessageFactory as _

    # The commented out messages are now part of the general purpose RE validator.
    # See config.py for initial values.
    newMessages = {
        'isDecimal' :   _(u'pfg_isDecimal', u'This field requires a decimal value.'),
        'isInt' :       _(u'pfg_isInt', u'This field requires an integer value.'),
        'isPrintable' : _(u'pfg_isPrintable', u'This value contains unprintable characters.'),
        'isSSN' :       _(u'pfg_isSSN', u'This is not a well formed SSN.'),
        # 'isUSPhoneNumber' : _(u'pfg_isUSPhoneNumber', u'This is not a valid us phone number.'),
        # 'isInternationalPhoneNumber' : _(u'pfg_isInternationalPhoneNumber', u'This is not a valid international phone number.'),
        # 'isZipCode' :   _(u'pfg_isZipCode', u'This is not a valid zip code.'),
        # 'isURL' :       _(u'pfg_isURL', u'This is not a valid url.'),
        # 'isEmail' :     _(u'pfg_isEmail', u'This is not a valid email address.'),
        'isMailto' :    _(u'pfg_isMailto', u'This is not a valid mailto: url.'),
        'isUnixLikeName' : _(u'pfg_isUnixLikeName', u'This name is not a valid identifier.'),
        'isChecked' :   _(u'pfg_isChecked', u'This box must be checked.'),
        'isUnchecked' :   _(u'pfg_isUnchecked', u'This box must be unchecked.'),
        'isValidDate' :   _(u'pfg_isValidDate', u'The date entered was invalid.'),
        }
        
    newRequiredMessage = _(u'pfg_isRequired', u'This field is required.')

    def cleanupMessage(original, context, instance):
        """ Where original is the message from the Archetypes validator,
            return an improved, translatable message if available.
        """

        if type(original) in StringTypes:
            if original.find('is required, please correct.') > 0:
                return newRequiredMessage
            else:
                mo = verrorRE.match(original)
                if mo:
                    term = mo.groups()[0]

                    if term.find('pfgv_') == 0:
                        # this is one of the customizable pfg tool validators
                        fgt = getToolByName(instance, 'formgen_tool')
                        sv = fgt.stringValidators.get(term[5:])
                        if sv:
                            # this is already a messagestr
                            return sv['errmsg']
                        
                    nm = newMessages.get(term)
                    if nm:
                        # this is one of the simple messages for which we have a replacement
                        return nm
                    
                    if term == 'inExNumericRange':
                        mo = enrMessageRE.match(original)
                        if mo:
                            return _(u'pfg_not_number', u'Please enter a number here.')
                        mo = enrSmallMessageRE.match(original)
                        if mo:
                            groups = mo.groups()
                            return _(u'pfg_number_too_small',
                                     u"'${value}' is too small. Must be at least ${min}.",
                                     mapping={ 'value':groups[0], 'min':groups[1] })
                        mo = enrLargeMessageRE.match(original)
                        if mo:
                            groups = mo.groups()
                            return _(u'pfg_number_too_large',
                                     u"'${value}' is too large. Must be no greater than ${max}.",
                                     mapping={ 'value':groups[0], 'max':groups[1] })
                    elif term == 'isNotTooLong':
                        mo = mlTooLongRE.match(original)
                        if mo:
                            groups = mo.groups()
                            return _(u'pfg_too_long',
                                     u"'Entry too long. It should be no more than ${max} characters.",
                                     mapping={ 'max':groups[0] })
                            
                        

        return original

else:
    # don't have Plone 2.5+; don't do anything too ambitious

    newRequiredMessage = 'This field is required.'

    def cleanupMessage(original, context, instance):

        if type(original) in StringTypes:
            if original.find('is required, please correct.') > 0:
                return newRequiredMessage
            else:
                mo = verrorRE.match(original)
                if mo:
                    term = mo.groups()[0]
                    
                    if term.find('pfgv_') == 0:
                        # this is one of the customizable pfg tool validators
                        fgt = getToolByName(instance, 'formgen_tool')
                        sv = fgt.stringValidators.get(term[5:])
                        if sv:
                            return sv['errmsg']

        return original




from zope.interface import Interface

# This is just a marker inteface; PFG identifies adapters by asking
# contained objects if they provide this interface.
#
# Note that marking an object type with this interface 
# does not automatically put it on the "add" menu. That
# needs to be done via portal_types.

class IPloneFormGenActionAdapter(Interface):
    """actionAdapter marker interface
    """
